const { plugin: { pluginObjectTypes, pluginScope } } = require('@kl/constants');
const lo = require('lodash');
const path = require('path');
const fs = require('fs');
const consts = require('./consts');
const kswsTypes = require('./types');
const apiUtils = require('./common/open_api');
const apiUsers  = require('./common/users');
const apiEvents  = require('./common/events');
const apiDc  = require('./common/dcList');
const apiGuiCalls  = require('./common/guiCalls');

const licType_folder = {
 1:"select",
 2:"advanced",
 3:"enterprise"
};

function PluginServerAPI (kscApi, params) {
  const tempDataStore = (params && params.tempDataStore)  ?  params.tempDataStore : {};
  const policy = require('./converters/policy/index')();
  const policyProfile = require('./converters/policy_profile/index')();
  const settings = require('./converters/settings/index')();
  const tasksGroup = require('./converters/tasks/index.js')();
  const tasksLocal = require('./converters/local_tasks/index.js')();
  const distributive = require('./converters/distributive/index')();

  async function getFormModel ({ context: { object: { id, type, parentId }, scope }, form, type: taskType }) {
        console.log(consts.logProductPref + ' getFormModel ' + JSON.stringify(arguments));
        let data = { };
        switch (type) {
            case pluginObjectTypes.policy:
                data = await policy.getPolicyData(kscApi, id, parentId, form, tempDataStore);
                break
            case pluginObjectTypes.profile:
                data = await policyProfile.getData(kscApi, id, parentId, form, tempDataStore);
                break
            case pluginObjectTypes.task:
                data = await tasksGroup.getTaskData ({kscApi, taskId: id, hostId: false, taskType, formType: form, tDataStore:tempDataStore});
                break
            case pluginObjectTypes.taskInitial:
                let defData = await getTaskDefaults({taskType, scope});
                data = await tasksGroup.getWizardTaskData({kscApi, taskType, formType: form, defData, tDataStore:tempDataStore});
                break
            case pluginObjectTypes.localTask:
                data = await tasksLocal.getTaskData(kscApi, id, parentId, taskType, form, type, tempDataStore);
                break
            case pluginObjectTypes.settings:
                data = await settings.getSettingsData(kscApi, form, parentId, type, tempDataStore);
                break
            case pluginObjectTypes.distributive:
                data = await distributive.getPackageData ({kscApi, packageId: id, form});
                break
        }
        return data
  }

  async function setFormModel ({ context: { scope, object: { type, parentId } }, form, taskType }, data) {
      console.log(consts.logProductPref + ' setFormModel ' + JSON.stringify(arguments))
      switch (type) {
          case pluginObjectTypes.policy:
              await policy.updatePolicyData(kscApi, form, data, tempDataStore);
              break
          case pluginObjectTypes.profile:
              await policyProfile.updateData(kscApi, form, data, tempDataStore);
              break
          case pluginObjectTypes.task:
              if (scope === pluginScope.editObject)
                  await tasksGroup.updateTaskData ({kscApi, modelData: data, taskType, formType: form, tDataStore:tempDataStore})
              break
          case pluginObjectTypes.taskInitial:
              await tasksGroup.updateTaskData ({kscApi, modelData: data, taskType, formType: form, tDataStore:tempDataStore});
              break
          case pluginObjectTypes.localTask:
              if (scope === pluginScope.editObject)
                  await tasksLocal.updateTaskData ({kscApi, modelData: data, taskType, formType: form, hostId: parentId, tDataStore:tempDataStore});
              break
          case pluginObjectTypes.settings:
              await settings.updateSettingsData(kscApi, form, data, tempDataStore);
              break
          case pluginObjectTypes.distributive:
              data = await distributive.savePackageData ({kscApi, modelData: data, form});
              break
      }
      return true
  }

  async function beforeApplyChanges ({ object: { id, type }, scope }) {
      console.log(consts.logProductPref + ' beforeApplyChanges ' + JSON.stringify(arguments));
      switch (type) {
          case pluginObjectTypes.policy:
              policy.commitPolicyData(kscApi, scope);
              break
          case pluginObjectTypes.profile:
              policyProfile.commitData(kscApi, scope);
              break
          case pluginObjectTypes.settings:
              settings.commitSettingsData(kscApi);
              break
      }
  }

  async function applyChanges ({ object: { id, type, parentId }, scope }) {
      console.log(consts.logProductPref + ' applyChanges ' + JSON.stringify(arguments));
      switch (type) {
          case pluginObjectTypes.policy:
              await policy.savePolicyData(kscApi, id, scope, tempDataStore);
              break
          case pluginObjectTypes.profile:
              await policyProfile.saveData(kscApi, id, parentId, tempDataStore);
              break
          case pluginObjectTypes.settings:
              await settings.saveSettingsData(kscApi, id, tempDataStore);
              break
      }
  }

  /**
   * Метод для получения данных от плагина по имени файла
   * @param fileName - имя файла с запрашиваемыми данными
   * @returns {Promise<string | Buffer>} - текстовое содержимое файла
   */
  async function getAssetByFileName ({fileName}) {
      const kscLicType = await getEnvKswsLicType();
      let filePath = path.join(__dirname, 'assets', fileName);
      if(kscLicType!==kswsTypes.NwcLicType.onPrem && fileName==='task-types.json') {
          filePath = path.join(__dirname, 'assets', licType_folder[kscLicType],fileName);
      }
      if(kscLicType!==kswsTypes.NwcLicType.onPrem && fileName==='events.json') {
          filePath = path.join(__dirname, 'assets', licType_folder[kscLicType],fileName);
      }

      try {
          runtime.logger.log(consts.logProductPref + ' Read assets file: ' + filePath );
          return fs.readFileSync(filePath, 'utf-8');
      } catch (error) {
          console.error(error);
      }
      return '';
  }

  /**
   * Запуск продукта на устройстве
   * @param {string} hostName - уникальное имя хоста
   * @return {Promise.<*>}
   */
  async function startProductOnHost ({ hostName }) {
    return hostName
  }

  /**
   * Остановка продукта на устройстве
   * @param {string} hostName - уникальное имя хоста
   * @return {Promise.<*>}
   */
  async function stopProductOnHost ({ hostName }) {
    return hostName
  }

  /**
   * Возвращает параметры задачи по-умолчанию
   * @param taskType - тип задачи
   * @param scope - контекст выполнения операции
   * @returns {Promise.<*>}
   */
  async function getTaskDefaults({ taskType, scope }) {
      console.log(consts.logProductPref + ' getTaskDefaults ' + JSON.stringify(arguments))
      switch (scope) {
          case pluginScope.newObject:
          case pluginScope.newQSWObject:
          case pluginScope.newCloudProtectionObject:
              const tasksRaw = await getAssetByFileName({ fileName: 'task-defaults.json' });
              const tasks = JSON.parse(tasksRaw);
              return tasks[taskType];
      }
      return {}
  }

  /**
   * Возвращает продуктовые настройки задачи
   * @param taskData - настройки задачи KSC
   * @param scope - контекст выполнения операции
   * @returns {Promise.<*>}
   */
  async function getTaskAdditionalParams ({ task, scope }) {
    console.log(consts.logProductPref + ' getTaskAdditionalParams ' + JSON.stringify(arguments));
    let data = {};
    switch (scope) {
          case pluginScope.newObject:
              data = await tasksGroup.getTaskAdditionalParams(tempDataStore);
              break
          case pluginScope.editObject:
              if(task.isLocalTask)
                  data = await tasksLocal.getTaskAdditionalParams(task.TASK_NAME, tempDataStore);
              else
                  data = await tasksGroup.getTaskAdditionalParams(tempDataStore);
              break
          case pluginScope.newQSWObject:
              data = task.TASK_ADDITIONAL_PARAMS;
              break
    }
    return data;
  }

  /**
   * Return task registration data
   * @param taskType - required task type
   * @returns {Promise.<object>}
   */
  async function getTaskRegistrationData ({ taskType }) {
    const taskTypesJson = await getAssetByFileName({ fileName: 'task-types.json' });
    const taskTypes = JSON.parse(taskTypesJson);
    const taskData = lo.find(taskTypes, { type: taskType });
    if (!taskData) {
      throw new Error('Unknown task type');
    }
    return taskData;
  }

  //
  // internal methods
  //

  async function callApi ({ command, params }) {
     const [ openApiInterface, openApiMethod] = command.split('.');
     const response = await kscApi[openApiInterface][openApiMethod](params);
     return response.result;
  }

  async function EnumLicenseKeys() {
      return await apiUtils.EnumCompatibleLicenseKeys(kscApi);
  }

   async function GetKeyData({serial}) {
        const response = await kscApi.LicenseKeys.GetKeyData({
            pKeyInfo: {'KLLIC_SERIAL': serial, 'KLLICSRV_KEYDATA': true}
        })
        return response.result;
   }

    async function IsKpsnMode() {
        return await apiUtils.IsSetKpsnMode(kscApi);
    }

    async function IsHostedMode() {
        return await apiUtils.IsSetHostedMode(kscApi);
    }

    async function getEnvKswsLicType() {
        return kswsTypes.NwcLicType.onPrem;
  /*
      const bIsHosted = await apiUtils.IsSetHostedMode(kscApi);
      if(!bIsHosted)
          return kswsTypes.NwcLicType.onPrem;
      const kswsLicType = process.env.KswsLicType;
      if(kswsLicType==='2' || kswsLicType===2)
          return kswsTypes.NwcLicType.BusinessAdvanced;
      if(kswsLicType==='3' || kswsLicType===3)
          return kswsTypes.NwcLicType.EnterpriseServer;
      return kswsTypes.NwcLicType.BusinessSelect;
*/
    }

    async function UpdatePolicyDataFromForm({ form, data }) {
        await policy.updatePolicyData(kscApi, form, data, tempDataStore);
    }

    async function EnumPlcConfigs(vUsedPlcIds) {
        return await apiUtils.EnumRegistryPlcConfigs(kscApi, vUsedPlcIds);
    }

    async function EnumNetListPlcConfigs(hostId, vUsedNetIds) {
        return await apiUtils.EnumNetworkListPlcConfigs(kscApi, hostId, vUsedNetIds);
    }

    async function EnumNetListFirmwares(NetId) {
        return await apiUtils.EnumNetworkListFirmwares(kscApi, NetId);
    }

    async function GetPlcFromRegistry(internalId) {
        return await apiUtils.GetPlcFromRegistry(kscApi, internalId);
    }

    async function UpdatePlcInRegistry(regCfg) {
        return await apiUtils.UpdatePlcFromRegistry(kscApi, regCfg);
    }

    async function DeletePlcFromRegistry(internalId) {
        return await apiUtils.DeletePlcFromRegistry(kscApi, internalId);
    }

    async function EnumNetListWiFi() {
        return await apiUtils.EnumNetworkListWiFi(kscApi);
    }

    async function SearchUsers(startPos, endPos, search) {
        return await apiUsers.SearchKscUsers(kscApi, startPos, endPos, search);
    }

    async function GetUsersBySid(sids) {
        return await apiUsers.GetKscUsersBySid(kscApi, sids);
    }

    async function binarySidToWinSid(sid) {
        return await kscApi.utils.binToSID({ binary: Buffer.from(sid, 'base64') });
    }

    async function winSidToBinarySid(sid) {
        return await kscApi.utils.SIDToBin({ sid });
    }

    async function SearchAcEvents(startPos, endPos, requestParams) {
        return await apiEvents.SearchAcKscEvents(kscApi, startPos, endPos, requestParams);
    }

    async function GetCryptPassword(pswrd) {
        return await apiUtils.CryptPassword(kscApi, pswrd);
    }

    async function SearchDcNetList(startPos, endPos, search) {
        return await apiDc.SearchKscDc(kscApi, startPos, endPos, search);
    }

    async function GetBlockedHostSessions(hostName) {
        return apiGuiCalls.GetBlockedHostSessions(kscApi, hostName);
    }

    async function RemoveOneBlockedHostSessions(hostName, hostData) {
        return apiGuiCalls.RemoveOneBlockedHostSessions(kscApi, hostName, hostData);
    }

    async function RemoveAllBlockedHostSessions(hostName) {
        return apiGuiCalls.RemoveAllBlockedHostSessions(kscApi, hostName);
    }

    return {
        getFormModel,
        setFormModel,
        beforeApplyChanges,
        applyChanges,
        getAssetByFileName,
        startProductOnHost,
        stopProductOnHost,
        getTaskAdditionalParams,
        getTaskDefaults,
        getTaskRegistrationData,
        internal: {
            EnumLicenseKeys,
            GetKeyData,
            IsKpsnMode,
            IsHostedMode,
            getEnvKswsLicType,
            UpdatePolicyDataFromForm,
            EnumPlcConfigs,
            EnumNetListPlcConfigs,
            EnumNetListFirmwares,
            GetPlcFromRegistry,
            UpdatePlcInRegistry,
            DeletePlcFromRegistry,
            EnumNetListWiFi,
            SearchUsers,
            GetUsersBySid,
            binarySidToWinSid,
            winSidToBinarySid,
            SearchAcEvents,
            GetCryptPassword,
            SearchDcNetList,
            GetBlockedHostSessions,
            RemoveOneBlockedHostSessions,
            RemoveAllBlockedHostSessions
        }
  }
}

module.exports = PluginServerAPI;