
const ShowForm = (formId, model, fOnOk, fValidate) => {
    const form = runtime.ui.form(formId);
    form.setModel(model);

    const onAccept = () => {
        if (fValidate && !fValidate(form))
            return;

        fOnOk(form);
        form.close();
    };

    form
        .show()
        .actions
        .clear()
        .addOk(onAccept)
        .addCancel()
        .show()
        .close()
};

const ShowFormEx = (formId, model, fOnAccept) => {
    const form = runtime.ui.form(formId);
    form.setModel(model);
    form
        .show()
        .actions
        .clear()
        .addOk(()=>{
            fOnAccept(form);
        })
        .addCancel(() => {
            form.close();
        })
        .show()
};

const ShowConfirmPopup = (titleId, textId, fConfirm) => {
    const localization = runtime.getSync('services.localization');
    runtime.ksc.trigger('showConfirmationPopup', {
        popupTitle: localization.get(titleId),
        confirmationText:localization.get(textId),
        okButtonText:localization.get('btns.Yes'),
        cancelButtonText:localization.get('btns.No'),
        onConfirm: () => { fConfirm(); },
        onCancel: () => {},
        onClose: () => {}
    });
}

const ShowMessageBox = (titleId, textId) => {
    const localization = runtime.getSync('services.localization');
    const cfg = {
        popupTitle: localization.get(titleId),
        confirmationText:localization.get(textId),
        okButtonText:localization.get('btns.Yes'),
        cancelButtonText:localization.get('btns.No'),
        hideCancelButton: true,
        onConfirm: () => {},
        onCancel: () => {},
        onClose: () => {}
    };
    runtime.ksc.trigger('showConfirmationPopup', cfg);
}

const isSid = (name) => {
    let result = name.match(/^S-1-[0-59]-\d{2}-\d{8,10}-\d{8,10}-\d{8,10}-[1-9]\d{3}/);
    if(!result){
        result = name.match(/^S-\d-(\d+-){1,14}\d+$/);
    }
    return !!(result && result.length > 0);
}

const GetLocalizedUsers = async (Users) => {
    if(Users===undefined || Users.length===0 )
        return '';

    const localization = runtime.getSync('services.localization');
    const firstUser = Users[0].User;
    const locRes = 'Global.WellKnownSids.' + firstUser;
    let out = isSid(firstUser) ? localization.get(locRes) : firstUser;
    if(locRes === out) {
        const binSid = await server.internal.winSidToBinarySid(firstUser);
        const users = await server.internal.GetUsersBySid([binSid]);
        out = users && users[0] ? users[0].ul_wstrDisplayName : firstUser;
    }
    if(Users.length > 1 ){
        const hided = Users.length;
        out += ' ... (' + hided.toString() + ')';
    }

    return out;
}

module.exports = {
    ShowForm,
    ShowFormEx,
    ShowConfirmPopup,
    ShowMessageBox,
    GetLocalizedUsers,
    isSid
}