const DEFAULT_DC_FILTER = '(dev_type<>13)';
const DEFAULT_DC_FIELDS = ['dev_id', 'dev_descr', 'dev_manuf', 'dev_name', 'dev_serialnum', 'dev_type', 'dev_vidpid'];

const EnumKscDc = async (kscApi, startPos, endPos, filter) => {
  try {
    const iterator = await kscApi.SrvView.ResetIterator({
      wstrViewName: 'DCDevicesList',
      wstrFilter: filter,
      vecFieldsToReturn: DEFAULT_DC_FIELDS
    });
    if (!iterator.result)
      return { rows: [], count: 0 };

    const nRecords = await kscApi.SrvView.GetRecordCount({wstrIteratorId:iterator.result});
    if (startPos >= nRecords.result){
      await kscApi.SrvView.ReleaseIterator({wstrIteratorId:iterator.result});
      return { rows: [], count: nRecords.result };
    }

    const recsRange = await kscApi.SrvView.GetRecordRange({
      wstrIteratorId: iterator.result,
      nStart: startPos,
      nEnd: (endPos < nRecords.result  ? endPos : nRecords.result)
    });
    await kscApi.SrvView.ReleaseIterator({wstrIteratorId:iterator.result});

    if(recsRange.result.KLCSP_ITERATOR_ARRAY) {
      return { rows: recsRange.result.KLCSP_ITERATOR_ARRAY, count:nRecords.result};
    }
  } catch (e) {
    runtime.logger.log('- kess_windows - : EnumSrvViewRecords failture. Error = ' + e);
  }
  return { rows: [], count: 0 };
}


const SearchKscDc = async (kscApi, startPos, endPos, search = []) => {
  let filter = DEFAULT_DC_FILTER;
  if (search) {
    const escapedSearch = String(search.replace('"', '\\22'));
    filter = `(&${DEFAULT_DC_FILTER}(|(dev_id = "*${escapedSearch}*")(dev_manuf = "*${escapedSearch}*")(dev_name = "*${escapedSearch}*")(dev_vidpid = "*${escapedSearch}*")))`;
  }
  return await EnumKscDc(kscApi, startPos, endPos, filter);
}

module.exports = {
  SearchKscDc,
}
