const consts = require('../consts.js');

const DatePeriodsEnum = {
    Total: 0,
    Year: 1,
    Month: 2,
    Week: 3,
    Day: 4,
};

const HourInMs = 1000*3600;

const pad = (number, length) => {
    let str = '' + number;
    while (str.length < length) {
        str = '0' + str;
    }
    return str;
}

const GetTimeFilter = (periodType) => {
    const now = new Date();
    let d = undefined;
    switch (periodType) {
        case DatePeriodsEnum.Total:
            return '';
        case DatePeriodsEnum.Day:
            d = new Date(now - HourInMs*24);
            break;
        case DatePeriodsEnum.Week:
            d = new Date(now - HourInMs*24*7);
            break;
        case DatePeriodsEnum.Month:
            d = new Date(now - HourInMs*24*31);
            break;
        case DatePeriodsEnum.Year:
            d = new Date(now - HourInMs*24*365);
            break;
    }
    return '(rise_time >= T"' + pad(d.getFullYear(),4) + '-' + pad(d.getMonth()+1, 2) + '-' + pad(d.getDate(),2) +' 00:00:00")';
}

const EnumKscEvents = async (kscApi, startPos, endPos, filter, FieldsToReturn) => {
    try {
        const iterator = await kscApi.SrvView.ResetIterator({
            wstrViewName: 'EventsSrvViewName',
            wstrFilter: filter,
            vecFieldsToReturn: FieldsToReturn
        });
        if (!iterator.result || iterator.result.PxgError)
            return { rows: [], count: 0 };

        const nRecords = await kscApi.SrvView.GetRecordCount({wstrIteratorId:iterator.result});
        if (startPos >= nRecords.result){
            await kscApi.SrvView.ReleaseIterator({wstrIteratorId:iterator.result});
            return { rows: [], count: nRecords.result };
        }

        const recsRange = await kscApi.SrvView.GetRecordRange({
            wstrIteratorId: iterator.result,
            nStart: startPos,
            nEnd: (endPos < nRecords.result  ? endPos : nRecords.result)
        });
        await kscApi.SrvView.ReleaseIterator({wstrIteratorId:iterator.result});

        if(recsRange.result.KLCSP_ITERATOR_ARRAY) {
            return { rows: recsRange.result.KLCSP_ITERATOR_ARRAY, count:nRecords.result};
        }
    } catch (e) {
        runtime.logger.log(consts.logProductPref + ': EnumSrvViewRecords failture. Error = ' + e);
    }
    return { rows: [], count: 0 };
}

const SearchAcKscEvents = async (kscApi, startPos, endPos, requestParams) => {
    let filterAnd = '(product_name="' + consts.wstrProduct + '")(product_version="' + consts.wstrVersion + '")';
    filterAnd = filterAnd + GetTimeFilter(requestParams.Interval);
    let filterOr = '';
    if(requestParams.TypeEvents === 1){
        filterOr = '(|(event_type="GNRL_EV_APPLICATION_LAUNCH_DENIED"))';
    } else if(requestParams.TypeEvents === 2){
        filterOr = '(|(event_type="GNRL_EV_APP_LAUNCH_TESTED_DENIED"))';
    } else {
        filterOr = '(|(event_type="GNRL_EV_APPLICATION_LAUNCH_DENIED")(event_type="GNRL_EV_APP_LAUNCH_TESTED_DENIED"))';
    }

    const filter = `(&${filterAnd}${filterOr})`;
    const records = await EnumKscEvents(kscApi, startPos, endPos, filter, ['GNRL_EA_DESCRIPTION']);
    return records;
}

module.exports = {
    SearchAcKscEvents
}