const { helpers } = require('@kl/openapi-client');
const { TextDecoder } =  require('util');
const consts = require('../consts.js');
const hlp_t = require('../helpers_time.js');
const buffer = require('buffer');
const binContainer = require('./binaryContainer.js');

//AppGui Calls functions's names
const APP_GUI_CALL_GET_BLOCKED_HOSTS_LIST = 'GetBlockedHostsList';
//const APP_GUI_CALL_ADD_BLOCKED_HOST_TILL = 'AddBlockedHostTill';
//const APP_GUI_CALL_ADD_BLOCKED_HOST_FOR_PERIOD = 'AddBlockedHostForPeriod';
const APP_GUI_CALL_REMOVE_BLOCKED_HOST = 'RemoveBlockedHost';
const APP_GUI_CALL_REMOVE_ALL_BLOCKED_HOST = 'RemoveAllBlockedHost';
const APP_GUI_CALL_GET_HOST_PROCESS_LIST = 'GetHostProcessList';

const checkState = async (api, wstrActionGuid, connectionId) => {
    return (async function checkStateInner() {
        const result = await api.AsyncActionStateCheckerGateway.CheckActionState( { wstrActionGuid, connectionId } );

        if (result.result.bFinalized) {
            return result.result;
        } else {
            if (result.result.PxgError) {
                return result.result
            }
            else {
                await new Promise(resolve => setTimeout(resolve, result.result.lNextCheckDelay));
                return checkStateInner();
            }
        }
    })();
}

const getDataFromHost = async (kscApi, KLHST_WKS_HOSTNAME, szwCallName, pInData) => {
    try {
        const connectionId = await kscApi.utils.createHostGatewayConnection({ KLHST_WKS_HOSTNAME });
        const result = await kscApi.NagGuiCalls.CallConnectorAsync({
            szwProduct: consts.wstrProduct,
            szwVersion: consts.wstrVersion,
            szwCallName: szwCallName,
            pInData: pInData,
            connectionId
        });

        const { bSuccededFinalized, pStateData } = await checkState(kscApi, result.result.PxgRetVal, connectionId);

        const nClosed = await kscApi.utils.closeHostGatewayConnection({ KLHST_WKS_HOSTNAME });
        runtime.logger.log(consts.logProductPref + '. kscApi.utils.closeHostGatewayConnection = ' + nClosed);
        if (bSuccededFinalized) {
            return pStateData.KLCONN_GUI_CALL_RESULTS;
        }
    } catch (error) {
        runtime.logger.error(consts.logProductPref + '::getDataFromHost. Canot get data from host. Error = ' + error);
    }
    runtime.logger.log(consts.logProductPref + '::getDataFromHost. Canot get data from host');
    return { GuiCallFailure: true };
}

const GetBlockedHostSessions = async (kscApi, KLHST_WKS_HOSTNAME) => {
    const res = await getDataFromHost(kscApi, KLHST_WKS_HOSTNAME, APP_GUI_CALL_GET_BLOCKED_HOSTS_LIST, {});
    if(res.value && res.value &&  res.value['1']) {
        const blockedHosts =  res.value['1'].map(i => {
            const dateAt = new Date(hlp_t.GetDateUtc(i.value['3'].value));
            const dateTill = new Date(hlp_t.GetDateUtc(i.value['4'].value));
           return {
               display_host_name: i.value['1'],
               original_host_name: i.value['2'],
               block_at: dateAt.toUTCString(),
               block_till: dateTill.toUTCString(),
               host_id: i.value['5'],
               rowValue: i.value
           };
        });
        return blockedHosts;
    }
    return [];
}

const RemoveOneBlockedHostSessions = async (kscApi, KLHST_WKS_HOSTNAME, hostData) => {
    const inData = (hostData && hostData.rowValue) ? hostData.rowValue : {};
    const res = await getDataFromHost(kscApi, KLHST_WKS_HOSTNAME, APP_GUI_CALL_REMOVE_BLOCKED_HOST, inData);
    return res.GuiCallFailure ? false : true;
}

const RemoveAllBlockedHostSessions = async (kscApi, KLHST_WKS_HOSTNAME) => {
    const res = await getDataFromHost(kscApi, KLHST_WKS_HOSTNAME, APP_GUI_CALL_REMOVE_ALL_BLOCKED_HOST, {});
    return res.GuiCallFailure ? false : true;
}

const GetRunningHostProcesses = async (kscApi, KLHST_WKS_HOSTNAME) => {
    try {
        const result = await getDataFromHost(kscApi, KLHST_WKS_HOSTNAME, APP_GUI_CALL_GET_HOST_PROCESS_LIST, {});

        const data = helpers.serializer.deserialize(result);
        const data2 = binContainer.blobToJson(data.ProcessArray.value);
        const buffered = Buffer.from(data.ProcessArray.value, 'base64');
        const string1 = buffered.toString();
        let decoded = new TextDecoder('utf-16').decode(buffered);
        const string2 = decoded.toString();

        let bytes = Buffer.from(data.ProcessArray.value, 'utf-8');
        let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');

        let base64Str2 = bufStr16.toString('utf-8');

        const stringified = buffered.toString('utf-8');
        const res = JSON.parse(stringified);
//        log('Row incidentData: ', { stringified });
        return res;
    } catch (error) {
        runtime.logger.error(consts.logProductPref + '::GetRunningHostProcesses. Canot get data from host. Error = ' + error);
    }
    return [];
}

module.exports = {
    GetBlockedHostSessions,
    RemoveOneBlockedHostSessions,
    RemoveAllBlockedHostSessions,
    GetRunningHostProcesses
}
