const utils = require('../utils');

const OdsActionEnum = {Cure:1, Remove:2, Recommended:3, Report:4, Quarantine: 5, Skip: 7, Postpone:8};
const InfAA = {cure:0, cure_delete:1, delete:2, recommended:3, notify:100 };
const SuspAA = { quarantine: 0, delete:1, recommended:2, notify:100 };

const getOdsUIFields = (scanSett) => {
    let infAction = InfAA.notify;
    let suspAction = SuspAA.quarantine;

    switch(scanSett.InfectedFirstAction.ActionType)  {
        case OdsActionEnum.Cure:
            infAction = scanSett.InfectedSecondAction.ActionType===OdsActionEnum.Remove ? InfAA.cure_delete : InfAA.cure;
            break;
        case OdsActionEnum.Remove:
            infAction = InfAA.delete;
            break;
        case OdsActionEnum.Recommended:
            infAction = InfAA.recommended;
            break;
    }

    switch(scanSett.SuspiciousFirstAction.ActionType)  {
        case OdsActionEnum.Info:
        case OdsActionEnum.Skip:
        case OdsActionEnum.Report:
            suspAction = SuspAA.notify;
            break;
        case OdsActionEnum.Quarantine:
            suspAction = SuspAA.quarantine;
            break;
        case OdsActionEnum.Remove:
            suspAction = SuspAA.delete;
            break;
        case OdsActionEnum.Recommended:
            suspAction = SuspAA.recommended;
            break;
    }
    return {
        InfAction: infAction,
        SuspAction: suspAction,
        ExtList: scanSett.ScanExtensionMasks.join(),
        ExcludeMasks: scanSett.ExcludeMasks.join(),
        ExcludeThreats:scanSett.ExcludeThreats.join(),
        Bigger: scanSett.SizeLimit/(1024*1024)
    };
}

const UIFieldsToOdsSettings = (scanSett, uiFields) => {
    if(uiFields===undefined)
        scanSett;

    scanSett.ScanExtensionMasks = utils.normalizedArrayMasks(uiFields.ExtList);
    scanSett.ExcludeMasks = utils.normalizedArrayMasks(uiFields.ExcludeMasks);
    scanSett.ExcludeThreats = utils.normalizedArrayMasks(uiFields.ExcludeThreats);
    scanSett.SizeLimit = uiFields.Bigger*(1024*1024)

    switch (uiFields.InfAction) {
        case InfAA.cure:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Cure;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case InfAA.cure_delete:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Cure;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Remove;
            break;
        case InfAA.delete:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Remove;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case InfAA.recommended:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Recommended;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Recommended;
            break;
        case InfAA.notify:
        default:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Report;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Report;
            break;
    }

    switch (uiFields.SuspAction) {
        case SuspAA.quarantine:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Quarantine;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case SuspAA.delete:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Remove;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case SuspAA.notify:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Report;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case SuspAA.recommended:
        default:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Recommended;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Recommended;
            break;
    }

    return scanSett;
}

module.exports = {
    getOdsUIFields,
    UIFieldsToOdsSettings,
}