const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpSch = require('./TaskScheduleHelper.js');
const utilsFw = require('../../utils/fw.js');

const defPolicySections = ['FirewallPolicy','PredefinedTasksPolicy'];
const formAppRules = ['ApplicationRules'];
const formOutAppRules = ['OutboundApplicationRules'];
const formPortRules = ['OpenPorts'];
const formOutPortRules = ['OutboundOpenPorts'];
const formAdditional = ['EnableICMPv4', 'EnableICMPv6', 'InboundAction', 'OutboundAction'];

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.FirewallStartOptions);
    }
    if(formType!=='Main')
        return {};

    const appRulesIn = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.FirewallPolicy, formAppRules);
    const portRulesIn = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.FirewallPolicy, formPortRules);
    const data = {
        Form_Network_FirewallAppRules: {
            LocalTask: true,
            ApplicationRules : {
                ...appRulesIn.ApplicationRules,
                KLPRSS_Val:[ utilsFw.upgradeAppRulesPolicy(appRulesIn.ApplicationRules.KLPRSS_Val[0]), false]
            }
        },
        Form_Network_FirewallPortRules: {
            LocalTask: true,
            OpenPorts : {
                ...portRulesIn.OpenPorts,
                KLPRSS_Val:[ utilsFw.upgradePortRulesPolicy(portRulesIn.OpenPorts.KLPRSS_Val[0]), false]
            }
        },
        Form_Firewall_AppRules_Out: {
            LocalTask: true,
            OutboundApplicationRules:  {KLPRSS_Mnd: false, KLPRSS_Val:[[], false]},
            ...hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.FirewallPolicy, formOutAppRules)
        },
        Form_Firewall_PortRules_Out: {
            LocalTask: true,
            OutboundOpenPorts: {KLPRSS_Mnd: false, KLPRSS_Val:[[], false]},
            ...hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.FirewallPolicy, formOutPortRules)
        },
        Form_Network_FirewallAdditional: {
            LocalTask: true,
            EnableICMPv4: {KLPRSS_Mnd: false, KLPRSS_Val:[false, false]},
            EnableICMPv6: {KLPRSS_Mnd: false, KLPRSS_Val:[false, false]},
            InboundAction: {KLPRSS_Mnd: false, KLPRSS_Val:[1, false]},
            OutboundAction: {KLPRSS_Mnd: false, KLPRSS_Val:[0, false]},
            ...hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.FirewallPolicy, formAdditional)
        }
    };
    return data;
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    else if(formType==='Main') {
        TASK_ADDITIONAL_PARAMS.Settings = {
            ApplicationRules: changedData.Form_Network_FirewallAppRules.ApplicationRules.KLPRSS_Val[0].map(rule => {
                return utilsFw.toSettingsAppRule(rule);
            }),
            EnableICMPv4: changedData.Form_Network_FirewallAdditional.EnableICMPv4.KLPRSS_Val[0],
            EnableICMPv6: changedData.Form_Network_FirewallAdditional.EnableICMPv4.KLPRSS_Val[0],
            Groups: TASK_ADDITIONAL_PARAMS.Settings.Groups,
            InboundAction: changedData.Form_Network_FirewallAdditional.InboundAction.KLPRSS_Val[0],
            OpenPorts: changedData.Form_Network_FirewallPortRules.OpenPorts.KLPRSS_Val[0].map(rule => {
                return utilsFw.toSettingsPortRule(rule);
            }),
            OutboundAction: changedData.Form_Network_FirewallAdditional.OutboundAction.KLPRSS_Val[0],
            OutboundApplicationRules: changedData.Form_Firewall_AppRules_Out.OutboundApplicationRules.KLPRSS_Val[0].map(rule => {
                return utilsFw.toSettingsAppRule(rule);
            }),
            OutboundOpenPorts: changedData.Form_Firewall_PortRules_Out.OutboundOpenPorts.KLPRSS_Val[0].map(rule => {
                return utilsFw.toSettingsPortRule(rule);
            }),
            RemoteAdmin: TASK_ADDITIONAL_PARAMS.Settings.RemoteAdmin,
            Services: TASK_ADDITIONAL_PARAMS.Settings.Services,
            __VersionInfo: [1, 3]
        };
    }
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formAdditional.concat(['ApplicationRules', 'OutboundApplicationRules', 'OpenPorts','OutboundOpenPorts']);
    activePolicyCache.FirewallPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.FirewallPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['FirewallStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
