const lo     = require('lodash');
const hlp    = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpIcap = require('../../utils/icap.js');
const hlpSch = require('./TaskScheduleHelper.js');

const defPolicySections = ['ICAPScannerPolicy','PredefinedTasksPolicy']
const formCommonValues = ['ServerPort','RespModeUri','UseHeuristicAnalyzer','HeuristicAnalyzerLevel', 'UseKsn','UseSandbox']
const formScopeValues = ['ScanScope']

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.ICAPSCannerStartOptions);
    }

    if(formType!=='Main')
        return {}

    let data = {
        Form_Nas_IcapCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ICAPScannerPolicy,formCommonValues),
        Form_Nas_IcapProtLevel: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ICAPScannerPolicy,formScopeValues)
    }
    // Protection Level
    const scanSetts = data.Form_Nas_IcapProtLevel.ScanScope.KLPRSS_Val[0][0].ScanSettings;
    data.Form_Nas_IcapProtLevel.ScanSettings = scanSetts
    data.Form_Nas_IcapProtLevel.uiFields = hlpIcap.getIcapProtLevelUiFields(scanSetts);

    data.Form_Nas_IcapCommon.LocalTask = true;
    data.Form_Nas_IcapProtLevel.LocalTask = true;
    return data
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    else if(formType==='Main') {
        // common
        const IcapCommon = hlp.ItemToLocalSettings(changedData.Form_Nas_IcapCommon);
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(IcapCommon, formCommonValues))
        // Protection Level
        const newScanScope = hlpIcap.IcapLocalScanSettingsFromUiFields(changedData.Form_Nas_IcapProtLevel);
        TASK_ADDITIONAL_PARAMS.Settings.ScanScope = [newScanScope];
    }
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formScopeValues);
    activePolicyCache.ICAPScannerPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.ICAPScannerPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['ICAPSCannerStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
