const lo = require('lodash');
const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpSch = require('./TaskScheduleHelper.js');
const hlpKsn = require('../../utils/ksn');

const defPolicySections = ['KsnServicePolicy','PredefinedTasksPolicy']
const formCommonValues = ['ActionOnDetect','UseSizeLimit','SizeLimit','UsageType']
const formEulaValues = ['ConfirmTermsAndConditions','UseUdsForFiles','UseUdsForUrls','SendStatistics','UseKpsn']
const formKpsnEulaValues = ['ConfirmTermsAndConditions','UseUdsForFiles','UseUdsForUrls','SendStatistics','UseKpsn','UseKmpService']
const formKmpValues = ['ConfirmTermsAndConditions','UseKmpService']

async function toViewEx(kscApi, TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.KsnSeriveStartOptions);
    }

    let data = {};
    switch (formType) {
        case 'Protection_KsnCommon':
            data = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.KsnServicePolicy,formCommonValues);
            data.ActionType = data.ActionOnDetect.KLPRSS_Val[0].ActionType.KLPRSS_Val
            if( hlpKsn.KsnActionEnum.Backup == data.ActionType )
                data.ActionType = hlpKsn.KsnActionEnum.Remove
            if( hlpKsn.KsnActionEnum.Skip == data.ActionType )
                data.ActionType = hlpKsn.KsnActionEnum.Report
            data.UseKscProxy = (data.UsageType.KLPRSS_Val[0] === hlpKsn.KsnProxyUsageEnum.Global) ? true : false
            break
        case 'Protection_KsnEula':
            data = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.KsnServicePolicy,formEulaValues);
            data.ConfirmKpsn = (data.ConfirmTermsAndConditions.KLPRSS_Val[0] && data.UseKpsn.KLPRSS_Val[0]);
            data.LocEula = await hlpKsn.getEula(kscApi);
            break
        case 'Protection_KpsnEula':
            data = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.KsnServicePolicy,formKpsnEulaValues);
            data.LocEula = await hlpKsn.getKpsnEula(kscApi);
            break
        case 'Protection_KmpEula':
            data = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.KsnServicePolicy,formKmpValues);
            data.LocEula = await hlpKsn.getKmpEula(kscApi);
            break
    }
    data.LocalTask = true;
    return data
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }

    switch (formType) {
        case 'Protection_KsnCommon':
            hlp.ItemToLocalSettings(changedData);
            changedData.ActionOnDetect.ActionType = changedData.ActionType
            changedData.UsageType = (changedData.UseKscProxy === true) ? hlpKsn.KsnProxyUsageEnum.Global : hlpKsn.KsnProxyUsageEnum.None;
            Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(changedData, formCommonValues))
            break
        case 'Protection_KsnEula':
            changedData.UseKpsn.KLPRSS_Val[0] = false;
            hlp.ItemToLocalSettings(changedData);
            Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(changedData, formEulaValues));
            break
        case 'Protection_KpsnEula':
            changedData.ConfirmTermsAndConditions.KLPRSS_Val[0] = changedData.ConfirmKpsn;
            changedData.UseKpsn.KLPRSS_Val[0] = changedData.ConfirmKpsn;
            changedData.UseUdsForFiles.KLPRSS_Val[0] = changedData.ConfirmKpsn;
            changedData.UseUdsForUrls.KLPRSS_Val[0] = changedData.ConfirmKpsn;
            changedData.SendStatistics.KLPRSS_Val[0] = changedData.ConfirmKpsn;
            changedData.UseKmpService.KLPRSS_Val[0] = changedData.ConfirmKpsn;
            hlp.ItemToLocalSettings(changedData);
            Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(changedData, formEulaValues));
            break
        case 'Protection_KmpEula':
            hlp.ItemToLocalSettings(changedData);
            Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(changedData, formKmpValues))
            break
    }
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formKpsnEulaValues);
    activePolicyCache.KsnServicePolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.KsnServicePolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['KsnSeriveStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toViewEx,
    toRaw,
    RbacToPolicyCache
};
