const lo     = require('lodash');
const hlp    = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const utilNetApp = require('../../utils/netapp');
const hlpSch = require('./TaskScheduleHelper.js');

const defPolicySections = ['NetAppMonitorPolicy','PredefinedTasksPolicy']
const formCommonValues = ['UseHeuristicAnalyzer', 'HeuristicAnalyzerLevel','NetAppConnectAsInfo',
    'CheckFilerStatusPeriod', 'AttemptFilerReconnectThreshold','ClearFilerCacheWhenAvBasesUpdated',
    'UseKsn', 'UseTrustedZoneSettings','UseSandbox']
const formScopeValues = ['ScanScope', 'ExcludedFromScanScope'];

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.NetAppTaskStartOptions);
    }
    if(formType!=='Main')
        return {}

    let data = {
        Form_Nas_NetAppCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.NetAppMonitorPolicy,formCommonValues),
        Form_Nas_NetAppScope: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.NetAppMonitorPolicy,formScopeValues)
    }
    data.Form_Nas_NetAppCommon = utilNetApp.toViewNetAppPswrd(data.Form_Nas_NetAppCommon);
    data.Form_Nas_NetAppCommon.LocalTask = true;
    data.Form_Nas_NetAppScope.Areas = utilNetApp.toViewNetAppScanScope(data.Form_Nas_NetAppScope);
    data.Form_Nas_NetAppScope.LocalTask = true;
    return data
}

async function toRawEx(kscApi, TASK_ADDITIONAL_PARAMS, changedData, formType, hostId) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    if(formType!=='Main')
        return;
    // common
    changedData.Form_Nas_NetAppCommon = await utilNetApp.toRawNetAppSettPswrd(changedData.Form_Nas_NetAppCommon, kscApi, hostId);
    const netAppCommon = hlp.ItemToLocalSettings(changedData.Form_Nas_NetAppCommon);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(netAppCommon, formCommonValues))
    // scope
    const netAppScope = utilNetApp.toRowNetAppScanScopeSetts(changedData.Form_Nas_NetAppScope);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(netAppScope, formScopeValues));
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formScopeValues);
    activePolicyCache.NetAppMonitorPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.NetAppMonitorPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['NetAppTaskStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRawEx,
    RbacToPolicyCache
};
