const lo = require('lodash');
const crypto = require('crypto');
const buffer = require('buffer');
const hlp = require('../../../../helpers')


const sections = ['BLPolicy', 'PasswordPolicy']
const formValues = ['UseSelfProtection','OdsUseSelfRecovery', 'OdsSelfRecoveryAttempts', 'DontStartTasksIfNoSupply', 'StopScansIfNoSupply']

function toView (policiesCache) {
    let data = hlp.customToView(policiesCache.BLPolicy.Policy.KLPRSS_Val, formValues)
    data.PasswordSett = policiesCache.PasswordPolicy.Policy;
    data.NewPassword='';
    data.PasswordWasSet = data.PasswordSett.KLPRSS_Val.PasswordHash.KLPRSS_Val[0]!=='' ? true : false
    return data;
}

function toRaw (changedData, policiesCache) {
    if( changedData.PasswordSett.KLPRSS_Val.PasswordSalt.KLPRSS_Val[0]==='' ){
        const buf = Buffer.alloc(64);
        changedData.PasswordSett.KLPRSS_Val.PasswordSalt.KLPRSS_Val[0] = crypto.randomFillSync(buf).toString('hex');
    }
    if(changedData.PasswordSett.KLPRSS_Val.UsePassword.KLPRSS_Val[0]===true && changedData.NewPassword!==''){
        const saltBuff = Buffer.from(changedData.PasswordSett.KLPRSS_Val.PasswordSalt.KLPRSS_Val[0], 'hex');
        const pswBytes = Buffer.from(changedData.NewPassword, 'utf-8');
        const pswBytes16 = buffer.transcode(pswBytes, 'utf8', 'utf16le');
        const pswrd = crypto.pbkdf2Sync(pswBytes16, saltBuff, 1000, 64, 'sha512');
        if(pswrd){
            let hash = pswrd.toString('hex');
            changedData.PasswordSett.KLPRSS_Val.PasswordHash.KLPRSS_Val[0] = hash.toUpperCase();
        }

    }
    else{
        changedData.PasswordSett.KLPRSS_Val.PasswordHash.KLPRSS_Val[0] = '';
    }

    Object.assign(policiesCache.PasswordPolicy.Policy, changedData.PasswordSett);
    Object.assign(policiesCache.BLPolicy.Policy.KLPRSS_Val, lo.pick(changedData, formValues) );
}

module.exports = {
    sections,
    toView,
    toRaw
};
