const lo = require('lodash');
const hlp = require('../../../../helpers');
const hlpSchedule = require('../Schedule/schedule.js');
const utils    = require('../../../../utils');
const utilsPep = require('../../../utils/pep.js');

const sections = ['ApplicationControlPolicy','PredefinedTasksPolicy'];
const formCommonValues = ['EnforsmentMode','UseCache','BlockInterperetesWhenCliEmpty',
    'UseBinaryRules','MonitorDllLoading','UseScriptOrMsiRules',
    'BlockUntrustedApplication', 'AllowTrustedApplication','AllowUserOrGroups'];
const formRulesValues = ['RuleUsageStrategy','RuleCollection','AdditionalRuleCollection'];
const formSdcValues = ['UseTrustedInstaller', 'TrustWindowsInstaller', 'TrustBackgroundIntelligentTransferService', 'TrustedInstallers'];

function toView (policiesCache) {
    const policyAc = policiesCache.ApplicationControlPolicy.Policy.KLPRSS_Val;
    let data = {
        Form_Control_PepCommon: hlp.customToView(policyAc, formCommonValues),
        Form_Control_PepRules: hlp.customToView(policyAc, formRulesValues),
        Form_Control_PepSdc: hlp.customToView(policyAc, formSdcValues),
        Form_Policy_AppCtrlSchedule: hlpSchedule.toView(policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.ApplicationControlStartOptions)
    }

    data.Form_Control_PepCommon.Users = utils.toViewArray(policyAc.AllowUserOrGroups.KLPRSS_Val[0], 'User');
    data.Form_Control_PepRules.uiFields = utilsPep.toViewRules(data.Form_Control_PepRules);
    return data;
}

function toRaw (changedData, policiesCache) {
    // common
    Object.assign( policiesCache.ApplicationControlPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Control_PepCommon, formCommonValues));
    policiesCache.ApplicationControlPolicy.Policy.KLPRSS_Val.AllowUserOrGroups.KLPRSS_Val[0] = utils.toRawArray(changedData.Form_Control_PepCommon.Users,'User');
    // rules
    changedData.Form_Control_PepRules = utilsPep.toRowRules(changedData.Form_Control_PepRules);
    Object.assign(policiesCache.ApplicationControlPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Control_PepRules, formRulesValues));
    // sdc
    Object.assign(policiesCache.ApplicationControlPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Control_PepSdc, formSdcValues));

    // schedule
    hlpSchedule.toRaw(changedData.Form_Policy_AppCtrlSchedule, policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.ApplicationControlStartOptions);
}

module.exports = {
    sections,
    toView,
    toRaw
};
