const lo = require('lodash');
const hlp = require('../../../../helpers');
const hlpSchedule = require('../Schedule/schedule.js');
const utilsFw = require('../../../utils/fw.js');

const sections = ['FirewallPolicy', 'PredefinedTasksPolicy'];
const formAppRules = ['ApplicationRules'];
const formOutAppRules = ['OutboundApplicationRules'];
const formPortRules = ['OpenPorts'];
const formOutPortRules = ['OutboundOpenPorts'];
const formAdditional = ['EnableICMPv4', 'EnableICMPv6', 'InboundAction', 'OutboundAction'];

function toView (policiesCache) {
    const InAppRules = utilsFw.upgradeAppRulesPolicy(policiesCache.FirewallPolicy.Policy.KLPRSS_Val.ApplicationRules.KLPRSS_Val[0]);
    const InOpenPorts = utilsFw.upgradePortRulesPolicy(policiesCache.FirewallPolicy.Policy.KLPRSS_Val.OpenPorts.KLPRSS_Val[0]);
    const additional = hlp.customToView(policiesCache.FirewallPolicy.Policy.KLPRSS_Val, formAdditional);
    const OutAppRules = hlp.customToView(policiesCache.FirewallPolicy.Policy.KLPRSS_Val, formOutAppRules);
    const OutPortRules = hlp.customToView(policiesCache.FirewallPolicy.Policy.KLPRSS_Val, formOutPortRules);

    return {
        Form_Network_FirewallAppRules: {
            ApplicationRules : {
                ...policiesCache.FirewallPolicy.Policy.KLPRSS_Val.ApplicationRules,
                KLPRSS_Val:[ InAppRules, false]
            }
        },
        Form_Firewall_AppRules_Out: OutAppRules ? OutAppRules : {KLPRSS_Mnd: false, KLPRSS_Val:[[], false]},
        Form_Network_FirewallPortRules: {
            OpenPorts : {
                ...policiesCache.FirewallPolicy.Policy.KLPRSS_Val.OpenPorts,
                KLPRSS_Val:[ InOpenPorts, false]
            }
        },
        Form_Firewall_PortRules_Out: OutPortRules ? OutPortRules : {KLPRSS_Mnd: false, KLPRSS_Val:[[], false]},
        Form_Network_FirewallAdditional: additional ? additional : {
            EnableICMPv4: {KLPRSS_Mnd: false, KLPRSS_Val:[false, false]},
            EnableICMPv6: {KLPRSS_Mnd: false, KLPRSS_Val:[false, false]},
            InboundAction:{KLPRSS_Mnd: false, KLPRSS_Val:[1, false]},
            OutboundAction: {KLPRSS_Mnd: false, KLPRSS_Val:[0, false]}
        },
        Form_Policy_FirewallSchedule: hlpSchedule.toView(policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.FirewallStartOptions)
    };
}

function toRaw (changedData, policiesCache) {
    // correct inbound AppRules
    for(let item of changedData.Form_Network_FirewallAppRules.ApplicationRules.KLPRSS_Val[0]){
        let remoteAddress = item.RemoteAddresses.KLPRSS_Val.trim()
        if(remoteAddress==='')
            remoteAddress = '*'
        item.RemoteAddresses.KLPRSS_Val = remoteAddress
        item.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,2]}
    }
    // correct outbound AppRules
    for(let item of changedData.Form_Firewall_AppRules_Out.OutboundApplicationRules.KLPRSS_Val[0]){
        let remoteAddress = item.RemoteAddresses.KLPRSS_Val.trim()
        if(remoteAddress==='')
            remoteAddress = '*'
        item.RemoteAddresses.KLPRSS_Val = remoteAddress
        item.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,2]}
    }
    // correct inbound PortRules
    for(let item of changedData.Form_Network_FirewallPortRules.OpenPorts.KLPRSS_Val[0]){
        item.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,2]}
    }
    // correct outbound PortRules
    for(let item of changedData.Form_Firewall_PortRules_Out.OutboundOpenPorts.KLPRSS_Val[0]){
        item.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,2]}
    }
    // correct ICMP flags
    changedData.Form_Network_FirewallAdditional.EnableICMPv6 = changedData.Form_Network_FirewallAdditional.EnableICMPv4;
    // save
    Object.assign( policiesCache.FirewallPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Network_FirewallAppRules, formAppRules));
    Object.assign( policiesCache.FirewallPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Firewall_AppRules_Out, formOutAppRules));
    Object.assign( policiesCache.FirewallPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Network_FirewallPortRules, formPortRules));
    Object.assign( policiesCache.FirewallPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Firewall_PortRules_Out, formOutPortRules));
    Object.assign( policiesCache.FirewallPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Network_FirewallAdditional, formAdditional));
    // set latest settings version
    policiesCache.FirewallPolicy.Policy.KLPRSS_Val.__VersionInfo = {KLPRSS_Mnd: false, KLPRSS_Val: [1, 3]};

    hlpSchedule.toRaw(changedData.Form_Policy_FirewallSchedule, policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.FirewallStartOptions);
}

module.exports = {
    sections,
    toView,
    toRaw
};
