const lo = require('lodash');
const hlp = require('../../../../helpers');
const utilsOas = require('../../../utils/oas.js');
const hlpSchedule = require('../Schedule/schedule.js');

const sections = ['OasPolicy','PredefinedTasksPolicy'];
const formCommonValues = ['ScanByAccessType','UseHeuristicAnalyzer','HeuristicAnalyzerLevel','UseTrustedZoneSettings',
    'UseKsn','UseHostBlocker','UseSandbox','UseActiveDisinfection','UseDeepcheckOnProcessStart'];
const formScopeValues = ['ScanScope', 'ExcludedFromScanScope'];

function toView (policiesCache) {
    let data = {
        Form_Protection_RtpCommon: hlp.customToView(policiesCache.OasPolicy.Policy.KLPRSS_Val, formCommonValues),
        Form_Protection_RtpScope: hlp.customToView(policiesCache.OasPolicy.Policy.KLPRSS_Val, formScopeValues),
        Form_Policy_RtpSchedule: hlpSchedule.toView(policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.OasTaskStartOptions, true)
    }
    data.Form_Protection_RtpScope = utilsOas.toViewScanScope(data.Form_Protection_RtpScope);
    return data;
}

function toRaw (changedData, policiesCache) {
    if( policiesCache.OasPolicy && policiesCache.OasPolicy.Policy) {
        // common
        Object.assign( policiesCache.OasPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Protection_RtpCommon, formCommonValues));
        // scope
        changedData.Form_Protection_RtpScope = utilsOas.toRowScanScope(changedData.Form_Protection_RtpScope);
        Object.assign( policiesCache.OasPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Protection_RtpScope, formScopeValues));
    }
    // schedule
    if( policiesCache.PredefinedTasksPolicy && policiesCache.PredefinedTasksPolicy.Policy) {
        hlpSchedule.toRaw(changedData.Form_Policy_RtpSchedule, policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.OasTaskStartOptions);
    }
}

module.exports = {
    sections,
    toView,
    toRaw
};
