const hlp_t = require('../../../../helpers_time.js');
const scheduleDef = require('../../../../default/schedule.js');

function keysToView(section, keys, initialState = {}) {
    return keys.reduce((acc, key) => {
        acc[key] = section[key].KLPRSS_Val;
        return acc;
    }, initialState);
}

function keysToRow(section, keys, data) {
    for (let key of keys) {
        section[key].KLPRSS_Val = data[key];
    }
}

function GetSheduleModeByFrequency(freq)
{
    switch(freq) {
        case hlp_t.TaskScheduleFrequencyEnum.Never :
            return 1; // AlwaysOff;
        case hlp_t.TaskScheduleFrequencyEnum.AtApplicationStart :
            return 2; //AlwaysOn;
        default:
            break;
    }
    return 3; //UseSpecifiedSchedule;
}


const formValues = ['Enable', 'Frequency',
    'EnableDurationLimit','EnableSuspend','EnableDeactivate','RunMissedTasks','EnableRandomizationPeriod','RandomizationPeriod'];

////////////////////////////////////////////////////
//
function toView (ScheduleTaskSection, bCanBeSuspended) {
    if(ScheduleTaskSection === undefined ){
       ScheduleTaskSection = scheduleDef.defaultPolicySchedule();
    }

    let data = {};
    const ScheduleData = ScheduleTaskSection.KLPRSS_Val[0].Schedule.KLPRSS_Val;
    data = keysToView(ScheduleData, formValues);
    data.nHour = (ScheduleData.UnitCount.KLPRSS_Val === 0 ) ? 1 : ScheduleData.UnitCount.KLPRSS_Val;
    data.nDay  = data.nHour;
    data.nWeek = data.nHour;
    hlp_t.DaysOfWeektoView(data, ScheduleData.DaysOfWeek.KLPRSS_Val);
    data.ActivateDate = hlp_t.GetDateUtc(ScheduleData.ActivateTime.KLPRSS_Val.value);
    data.DeactivateTime = hlp_t.GetDateUtc(ScheduleData.DeactivateTime.KLPRSS_Val.value);
    data.StartTime = hlp_t.SecondsToTimeStr(ScheduleData.StartTime.KLPRSS_Val, true);
    data.SuspendFrom = hlp_t.SecondsToTimeStr(ScheduleData.SuspendFrom.KLPRSS_Val);
    data.SuspendTill = hlp_t.SecondsToTimeStr(ScheduleData.SuspendTill.KLPRSS_Val);
    data.DurationLimit = hlp_t.MinutesToTimeStr(ScheduleData.DurationLimit.KLPRSS_Val);
    data.RandomizeInMinutes = ScheduleData.RandomizationPeriod.KLPRSS_Val / 60;

    data.allScheduleData = ScheduleTaskSection;
    if(bCanBeSuspended){
        data.CanBeSuspended = bCanBeSuspended;
    }
    return data;
}

function toRaw (changedData, ScheduleTaskSection) {
    if(ScheduleTaskSection === undefined )
        return;

    // update Row from schedule
    let ScheduleData = changedData.allScheduleData.KLPRSS_Val[0].Schedule.KLPRSS_Val;
    keysToRow(ScheduleData, formValues, changedData);
    if(changedData.Frequency=== hlp_t.TaskScheduleFrequencyEnum.Hours )
        ScheduleData.UnitCount.KLPRSS_Val = changedData.nHour;
    else if(changedData.Frequency=== hlp_t.TaskScheduleFrequencyEnum.Days )
        ScheduleData.UnitCount.KLPRSS_Val = changedData.nDay;
    else if(changedData.Frequency=== hlp_t.TaskScheduleFrequencyEnum.Weeks )
        ScheduleData.UnitCount.KLPRSS_Val = changedData.nWeek;

    ScheduleData.DaysOfWeek.KLPRSS_Val = hlp_t.DaysOfWeektoRow(changedData);
    ScheduleData.ActivateTime.KLPRSS_Val.value = hlp_t.SetDateUtc(changedData.ActivateDate);
    ScheduleData.StartTime.KLPRSS_Val = hlp_t.TimeStrToSeconds(changedData.StartTime);
    ScheduleData.RandomizationPeriod.KLPRSS_Val = changedData.RandomizeInMinutes * 60;

    if(changedData.EnableDeactivate)
        ScheduleData.DeactivateTime.KLPRSS_Val.value = hlp_t.SetDateUtc(changedData.DeactivateTime);
    if(changedData.EnableSuspend){
        ScheduleData.SuspendFrom.KLPRSS_Val = hlp_t.TimeStrToSeconds(changedData.SuspendFrom);
        ScheduleData.SuspendTill.KLPRSS_Val = hlp_t.TimeStrToSeconds(changedData.SuspendTill);
    }
    if(changedData.EnableDurationLimit)
        ScheduleData.DurationLimit.KLPRSS_Val = hlp_t.TimeStrToMinutes(changedData.DurationLimit);

    Object.assign(changedData.allScheduleData.KLPRSS_Val[0].Schedule.KLPRSS_Val, ScheduleData);
    if( changedData.Enable===true ){
        changedData.allScheduleData.KLPRSS_Val[0].Mode.KLPRSS_Val = GetSheduleModeByFrequency(changedData.Frequency);
    }
    else{
        changedData.allScheduleData.KLPRSS_Val[0].Mode.KLPRSS_Val = 1; // AlwaysOff;
    }

    Object.assign( ScheduleTaskSection, changedData.allScheduleData);
}

module.exports = {
    toView,
    toRaw
};
