const lo = require('lodash');
const hlp = require('../../../../helpers');
const utilsTz = require('../../../utils/tz.js');

const sections = ['TrustedZoneSettings'];
const formTrustProcs = ['TrustAllBackupReadOperations', 'TrustedProcessesEnabled', 'TrustedProcesses'];

function toView (settCache, activePolicyCache) {
    const exclutionRules = hlp.GetMergedLocalSettingsKey(settCache.TrustedZoneSettings, activePolicyCache.TrustedZoneSettings, 'ExclutionRules');
    const trustProcs = hlp.localSettingsToView(settCache.TrustedZoneSettings, activePolicyCache.TrustedZoneSettings, formTrustProcs);
    const data = {
        Form_TrustedZones_Exclusions : {
            ExclutionRules : utilsTz.toViewExcludes(exclutionRules),
            LocalSettings: true,
            isExclutionsEditable : (exclutionRules.KLPRSS_ValLck || exclutionRules.KLPRSS_Mnd) ? false : true
        },
        Form_TrustedZones_Processes: {
            TrustAllBackupReadOperations: trustProcs.TrustAllBackupReadOperations,
            TrustedProcessesEnabled: trustProcs.TrustedProcessesEnabled,
            TrustedProcesses: utilsTz.toViewTrustedProcesses(trustProcs.TrustedProcesses),
            LocalSettings: true
        }
    };
    return data;
}

function toRaw (changedData, settCache) {
    // exclusions
    const exclusions = utilsTz.toRowLocalSettsExcludes(changedData.Form_TrustedZones_Exclusions.ExclutionRules);
    settCache.TrustedZoneSettings.ExclutionRules = exclusions;

    // trusted process
    changedData.Form_TrustedZones_Processes.TrustedProcesses = utilsTz.toRowTrustedProcesses(changedData.Form_TrustedZones_Processes.TrustedProcesses);
    hlp.ItemToLocalSettings(changedData.Form_TrustedZones_Processes);
    Object.assign(settCache.TrustedZoneSettings, lo.pick(changedData.Form_TrustedZones_Processes, formTrustProcs));
}

module.exports = {
    sections,
    toView,
    toRaw
};
