const fs = require('fs');
const path = require('path');
const hlpDefPolicy = require('../../default/policy.js');
const hlpDefSetts = require('../../default/settings.js');
const utils = require('../../utils.js');
const utilsCmn = require('./common.js');

const allAlertsList = [
    // common
    {id: 4111, severity:2},
    {id: 8194, severity:1},
    {id: 4196, severity:1},
    {id: 4101, severity:3},
    {id: 4102, severity:3},
    {id: 8227, severity:2},
    {id: 8202, severity:1},
    {id: 8204, severity:2},
    {id: 8206, severity:2},
    {id: 8200, severity:1},
    {id: 4136, severity:2},
    {id: 4144, severity:2},
    {id: 4135, severity:1},
    {id: 4143, severity:1},
    {id: 4197, severity:2},
    {id: 4866, severity:2},
    {id: 8210, severity:1},
    {id: 4110, severity:1},
    {id: 8240, severity:2},
    {id: 8241, severity:2},
    {id: 4115, severity:1},
    // dev control
    {id: 8289, severity:1},
    {id: 8295, severity:1},
    {id: 8291, severity:2},
    // app control
    {id: 8264, severity:1},
    {id: 8269, severity:1},
    {id: 8267, severity:2},
    // wifi
//    {id: 8981, severity:1},
//    {id: 8982, severity:2},
    // host blocker
//    {id: 4160, severity:2},
//    {id: 8277, severity:2},
    // plc
//    {id: 8707, severity:1},
    // log inspector
    {id: 8512, severity:1},
    {id: 8513, severity:1},
    {id: 8514, severity:1},
    {id: 8515, severity:1},
    {id: 8516, severity:1},
    {id: 8517, severity:1},
    {id: 8519, severity:1},
    {id: 8520, severity:1},
];

let _locEvents = {};

function readAlertsLocalization(loc) {
    try {
        const filePath = path.resolve(__dirname, `../../localization/alerts/${loc}.json`);
        const out = fs.readFileSync(filePath, 'utf8');
        return out!==undefined ? JSON.parse(out) : {};
    } catch (error) {
        console.log(error);
    }
    return {};
}

const GetAlert = (eventId, severity, locEvents) =>{
    let alert = hlpDefPolicy.defaultEventsAlertOption();
    alert.EventType.KLPRSS_Val = eventId;
    alert.EventSeverity.KLPRSS_Val = severity;
    if(eventId==4160 || eventId==8289){
        alert.NotifyTerminalUsers.KLPRSS_Val = true;
    }

    if(_locEvents!==undefined ){
        const ev_data =  _locEvents[eventId]!==undefined ? _locEvents[eventId] : _locEvents.def;
        const pref =  _locEvents.pref!==undefined ? _locEvents.pref : '';
        if( ev_data.AdminMessage!==undefined )
            alert.AdminMessage.KLPRSS_Val = pref + ' ' + ev_data.AdminMessage;
        if( ev_data.UserMessage!==undefined )
            alert.UserMessage.KLPRSS_Val = pref + ' ' + ev_data.UserMessage;
    }
    return alert;
}

const GetAlertSetts = (eventId, severity, locEvents) => {
    let alert = hlpDefSetts.defaultEventsAlertOptionSetts();
    alert.EventType = eventId;
    alert.EventSeverity = severity;
    if(eventId==4160 || eventId==8289){
        alert.NotifyTerminalUsers = true;
    }

    if(_locEvents!==undefined ){
        const ev_data =  _locEvents[eventId]!==undefined ? _locEvents[eventId] : _locEvents.def;
        const pref =  _locEvents.pref!==undefined ? _locEvents.pref : '';
        if( ev_data.AdminMessage!==undefined )
            alert.AdminMessage = pref + ' ' + ev_data.AdminMessage;
        if( ev_data.UserMessage!==undefined )
            alert.UserMessage = pref + ' ' + ev_data.UserMessage;
    }
    return alert;
}

const CreateDefaultEvents = (loc) => {
    _locEvents =  readAlertsLocalization(loc!==undefined ? loc : 'en-us');

    return allAlertsList.map(item=>{
        return GetAlert(item.id,item.severity);
    });
}

function CreateSettingEvents(loc, settCache) {
    _locEvents =  readAlertsLocalization(loc!==undefined ? loc : 'en-us');

    return allAlertsList.map(alert=>{
        const found = settCache.NotificationSettings.EventsAlertOptions.find(item =>{
            return item.EventType === alert.EventType;
        });
        return found ? found : GetAlertSetts(alert.id, alert.severity);
    });
}

function AlertToView(evt) {
    evt.uiFields = {
        AdminNetsendRecipients: evt.AdminNetsendSettings.KLPRSS_Val.Recipients.KLPRSS_Val.join(),
        AdminEmailRecipients: evt.EmailSettings.KLPRSS_Val.Recipients.KLPRSS_Val.join()
    };
    // cmd exec password
    const curCmdExecPswrd = evt.CmdExecSettings.KLPRSS_Val.ImpersonationPassword.KLPRSS_Val.value;
    evt.NewCmdExecPassword = '';
    evt.UseEmptyCmdExecPassword = (curCmdExecPswrd==='') ? true : false;
    evt.CmdExecPswrdWasSet = (curCmdExecPswrd==='') ? false : true;
    // email password
    const curSmtpPswrd = evt.EmailSettings.KLPRSS_Val.SmtpServerAuthPassword.KLPRSS_Val.value;
    evt.NewSmtpPassword = '';
    evt.UseEmptySmtpPassword = (curSmtpPswrd==='') ? true : false;
    evt.SmtpPswrdWasSet = (curSmtpPswrd==='') ? false : true;
    return evt;
}

async function AlertToRow(kscApi, evt) {
    evt.AdminNetsendSettings.KLPRSS_Val.Recipients.KLPRSS_Val = utils.normalizedArrayMasks(evt.uiFields.AdminNetsendRecipients);
    evt.EmailSettings.KLPRSS_Val.Recipients.KLPRSS_Val = utils.normalizedArrayMasks(evt.uiFields.AdminEmailRecipients);
    // cmd exec password
    if(evt.UseEmptyCmdExecPassword){
        evt.CmdExecSettings.KLPRSS_Val.ImpersonationPassword.KLPRSS_Val.value = '';
    }
    else if(evt.NewCmdExecPassword!=='') {
        const pswrdCmdExec = utilsCmn.getPswdrData(evt.NewCmdExecPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrdCmdExec })
        if(response.result!=='')
            evt.CmdExecSettings.KLPRSS_Val.ImpersonationPassword.KLPRSS_Val.value = response.result;
    }
    delete evt.UseEmptyCmdExecPassword;
    delete evt.NewCmdExecPassword;
    delete evt.CmdExecPswrdWasSet;

    // smtp
    if(evt.UseEmptySmtpPassword){
        evt.EmailSettings.KLPRSS_Val.SmtpServerAuthPassword.KLPRSS_Val.value = '';
    }
    else if(evt.NewSmtpPassword!=='') {
        const pswrdSmtp = utilsCmn.getPswdrData(evt.NewSmtpPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrdSmtp })
        if(response.result!=='')
            evt.EmailSettings.KLPRSS_Val.SmtpServerAuthPassword.KLPRSS_Val.value = response.result;
    }
    delete evt.UseEmptySmtpPassword;
    delete evt.NewSmtpPassword;
    delete evt.SmtpPswrdWasSet;

    return evt;
}

module.exports = {
    CreateDefaultEvents,
    CreateSettingEvents,
    AlertToView,
    AlertToRow
}