//const { helpers } = require('@kl/openapi-client')
const lo = require('lodash')
const fs = require('fs')
const path = require('path')

const consts = require('../../consts');
const hlp = require('../../helpers');
const defLoc = 'en-us';

const KsnActionEnum = {
    Remove: 1,
    Report: 2,
    Backup: 3,
    Skip: 4
}

const KsnProxyUsageEnum =
{
    None: 0,
    Global: 1,
    National: 2
}

function readFileEula(subFolder, loc) {
    try {
        const filePath = path.resolve(__dirname, `../../assets/${subFolder}/${loc}.txt`);
        const out = fs.readFileSync(filePath, 'utf8');
        runtime.logger.log(consts.logProductPref + ' readFileEula: ' + out);
        return out!==undefined ? out : '';
    } catch (error) {
        console.log(consts.logProductPref + ' Error = ' + error);
    }
    return ''
}

async function getEula(kscApi) {
    let loc = await kscApi.utils.getUILanguage();
    const eula = readFileEula('ksn', loc);
    return (eula==='' && loc!==defLoc) ? readFileEula('ksn', defLoc) : eula;
}

async function getKmpEula(kscApi) {
    let loc = await kscApi.utils.getUILanguage();
    const eula = readFileEula('kmp', loc);
    return (eula==='' && loc!==defLoc) ? readFileEula('kmp', defLoc) : eula;
}

async function getKpsnEula(kscApi){
    const lang = await kscApi.utils.getUILanguage();
    const tmp = lang.split('-');
    const loc = tmp.length > 0 ? tmp[0] : lang;
    let defKpsnEula = '';
    try{
        const eulas = await kscApi.KsnInternal.GetNKsnEulas()
        if(eulas.result!==undefined && eulas.result.pNKsnEulas!==undefined && eulas.result.pNKsnEulas.length > 0 ){
            for(let eula of eulas.result.pNKsnEulas){
                if(eula.value.NKsnEulaLoc===loc)
                    return eula.value.NKsnEula;
                if(eula.value.NKsnEulaLoc==='en')
                    defKpsnEula = eula.value.NKsnEula;
            }
        }
    }
    catch (error) {
    }
    return defKpsnEula!=='' ? defKpsnEula : await getEula(kscApi);
}

module.exports = {
    KsnActionEnum,
    KsnProxyUsageEnum,
    getEula,
    getKmpEula,
    getKpsnEula
}