const hlpDefPolicy = require('../../default/policy.js');
const hlp    = require('../../helpers');
const utils = require('../../utils');
const buffer = require('buffer');
const { TextDecoder, TextEncoder } =  require('util');

const NetAppActionEnum = {Cure:1, Remove:2, Recommended:3, Report:4, Quarantine:5,  Skip:7, Info:8 }
const InfAA = {cure:0, cure_delete:1, delete:2,recommended:3,report:4,skip:100 }
const SuspAA = { quarantine: 0, delete:1, recommended:2,report:3 }

function getPswdrData(pswrd) {
    let bytes = Buffer.from(pswrd + String.fromCharCode(0), 'utf-8');
    let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');
    return bufStr16.toString('base64');
}

function getNetAppScopeUiFields(scanSett, isIncluded) {
    let infAction = InfAA.report;
    const infSA = scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val;

    switch(scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val)  {
        case NetAppActionEnum.Cure:
            if(infSA===NetAppActionEnum.Report || infSA===NetAppActionEnum.Skip)
                infAction = InfAA.cure;
            if(infSA===NetAppActionEnum.Remove)
                infAction = InfAA.delete;
            break;
        case NetAppActionEnum.Remove:
            if(infSA===NetAppActionEnum.Report || infSA===NetAppActionEnum.Skip)
                infAction = InfAA.delete;
            break;
        case NetAppActionEnum.Recommended:
        case NetAppActionEnum.Quarantine:
            if(infSA===NetAppActionEnum.Report || infSA===NetAppActionEnum.Skip || infSA===NetAppActionEnum.Recommended)
                infAction = InfAA.recommended;
            break;
        case NetAppActionEnum.Report:
            if(infSA===NetAppActionEnum.Report || infSA===NetAppActionEnum.Skip)
                infAction = InfAA.report;
            break;
    }

    let suspAction = SuspAA.report;
    const suspSA = scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val;
    switch(scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val)  {
        case NetAppActionEnum.Quarantine:
            if(suspSA===NetAppActionEnum.Report || suspSA===NetAppActionEnum.Skip)
                suspAction = SuspAA.quarantine;
            break;
        case NetAppActionEnum.Cure:
        case NetAppActionEnum.Remove:
            if(suspSA===NetAppActionEnum.Report || suspSA===NetAppActionEnum.Skip)
                suspAction = SuspAA.delete;
            break;
        case NetAppActionEnum.Report:
            if(suspSA===NetAppActionEnum.Report || suspSA===NetAppActionEnum.Skip)
                suspAction = SuspAA.report;
            break;
        case NetAppActionEnum.Recommended:
            if(suspSA===NetAppActionEnum.Report || suspSA===NetAppActionEnum.Skip || suspSA===NetAppActionEnum.Recommended)
                suspAction = SuspAA.recommended;
            break;
    }

    const out = {
        Included:isIncluded,
        InfAction:infAction,
        SuspAction: suspAction,
        ExtList: scanSett.ScanExtensionMasks.KLPRSS_Val.join(),
        ExcludeMasks: scanSett.ExcludeMasks.KLPRSS_Val.join(),
        ExcludeThreats: scanSett.ExcludeThreats.KLPRSS_Val.join(),
        Bigger: (scanSett.SizeLimit.KLPRSS_Val/(1024*1024))
    };
    return out;
}

function updateNetAppScopeFromUiFields(scanSett, uiFields) {
    switch (uiFields.InfAction) {
        case InfAA.cure_delete:
            scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Cure;
            scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Remove;
            break;
        case InfAA.delete:
            scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Remove;
            scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            break;
        case InfAA.recommended:
            scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Recommended;
            scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Recommended;
            break;

        case InfAA.report:
            scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            break;
    }

    switch (uiFields.SuspAction) {
        case SuspAA.quarantine:
            scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Quarantine;
            scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            break;

        case SuspAA.delete:
            scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Remove;
            scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            break;

        case SuspAA.report:
            scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Report;
            break;

        case SuspAA.recommended:
            scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Recommended;
            scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = NetAppActionEnum.Recommended;
            break;
    }

    if(scanSett.ScanType.KLPRSS_Val===4){
        scanSett.ScanExtensionMasks.KLPRSS_Val = utils.normalizedArrayExts(uiFields.ExtList);
    }

    if(scanSett.UseExcludeMasks.KLPRSS_Val===true) {
        scanSett.ExcludeMasks.KLPRSS_Val = utils.normalizedArrayMasks(uiFields.ExcludeMasks);
    }
    if(scanSett.UseExcludeThreats.KLPRSS_Val===true) {
        scanSett.ExcludeThreats.KLPRSS_Val = utils.normalizedArrayMasks(uiFields.ExcludeThreats);
    }
    if(scanSett.UseSizeLimit.KLPRSS_Val===true) {
        scanSett.SizeLimit.KLPRSS_Val = uiFields.Bigger*1024*1024;
    }
}

function NetAppScopeSettingsFromUiFields(scanSett, uiFields) {
    switch (uiFields.InfAction) {
        case InfAA.cure_delete:
            scanSett.InfectedFirstAction.ActionType = NetAppActionEnum.Cure;
            scanSett.InfectedSecondAction.ActionType = NetAppActionEnum.Remove;
            break;
        case InfAA.delete:
            scanSett.InfectedFirstAction.ActionType = NetAppActionEnum.Remove;
            scanSett.InfectedSecondAction.ActionType = NetAppActionEnum.Report;
            break;
        case InfAA.recommended:
            scanSett.InfectedFirstAction.ActionType = NetAppActionEnum.Recommended;
            scanSett.InfectedSecondAction.ActionType = NetAppActionEnum.Recommended;
            break;

        case InfAA.report:
            scanSett.InfectedFirstAction.ActionType = NetAppActionEnum.Report;
            scanSett.InfectedSecondAction.ActionType = NetAppActionEnum.Report;
            break;
    }

    switch (uiFields.SuspAction) {
        case SuspAA.quarantine:
            scanSett.SuspiciousFirstAction.ActionType = NetAppActionEnum.Quarantine;
            scanSett.SuspiciousSecondAction.ActionType = NetAppActionEnum.Report;
            break;

        case SuspAA.delete:
            scanSett.SuspiciousFirstAction.ActionType = NetAppActionEnum.Remove;
            scanSett.SuspiciousSecondAction.ActionType = NetAppActionEnum.Report;
            break;

        case SuspAA.report:
            scanSett.SuspiciousFirstAction.ActionType = NetAppActionEnum.Report;
            scanSett.SuspiciousSecondAction.ActionType = NetAppActionEnum.Report;
            break;

        case SuspAA.recommended:
            scanSett.SuspiciousFirstAction.ActionType = NetAppActionEnum.Recommended;
            scanSett.SuspiciousSecondAction.ActionType = NetAppActionEnum.Recommended;
            break;
    }

    scanSett.ScanExtensionMasks = (scanSett.ScanType===4) ? utils.normalizedArrayExts(uiFields.ExtList) : [];
    scanSett.ExcludeMasks = (scanSett.UseExcludeMasks===true) ? utils.normalizedArrayMasks(uiFields.ExcludeMasks) : [];
    scanSett.ExcludeThreats = (scanSett.UseExcludeThreats===true) ? utils.normalizedArrayMasks(uiFields.ExcludeThreats) : [];
    if(scanSett.UseSizeLimit===true) {
        scanSett.SizeLimit = uiFields.Bigger*1024*1024;
    }

    return scanSett;
}

const GetNetAppScanScopeSettings = (item) => {
    const advActions = hlp.ItemToLocalSettings(item.ScanSettings.KLPRSS_Val.AdvancedActions.KLPRSS_Val);
    let scope = hlp.ItemToLocalSettings(item);
    NetAppScopeSettingsFromUiFields(scope.ScanSettings, scope.uiFields);
    scope.ScanSettings.AdvancedActions = advActions.map(a=>{
        const Act = {
            Actions: [{
                DetectCertainty: a.Actions.DetectCertainty,
                FirstAction: a.Actions.FirstAction,
                SecondAction: a.Actions.SecondAction,
                __VersionInfo: a.Actions.__VersionInfo
            }],
            Verdict: a.Verdict,
            __VersionInfo: a.__VersionInfo
        };
        return Act;
    });

    delete scope.uiFields;
    return scope;
}

function toViewNetAppPswrd(data) {
    const curPswrd = data.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationPassword.KLPRSS_Val.value;
    data.Login = data.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationLogin.KLPRSS_Val;
    data.NewPassword = '';
    data.UseEmptyPassword = (curPswrd==='') ? true : false;
    data.PasswordWasSet = (curPswrd==='') ? false : true;
    return data;
}

function toViewNetAppScanScope(data) {
    let areas = [];
    for (let scope of data.ScanScope.KLPRSS_Val[0]) {
        scope.uiFields = getNetAppScopeUiFields(scope.ScanSettings.KLPRSS_Val, true);
        areas.push(scope)
    }
    for (let item of data.ExcludedFromScanScope.KLPRSS_Val[0]) {
        let exclude = hlpDefPolicy.defaultNetAppScope();
        exclude.uiFields.Included = false;
        exclude.AreaName = item.AreaName;
        exclude.AreaType = item.AreaType;
        areas.push(exclude)
    }

    return areas;
}

async function toRawNetAppPswrd(changedData, kscApi) {
    changedData.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationLogin.KLPRSS_Val = changedData.Login;
    changedData.NetAppConnectAsInfo.KLPRSS_Val[0].EnableNetAppImpersonation.KLPRSS_Val = changedData.Login==='' ? false : true;

    if(changedData.UseEmptyPassword){
        changedData.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationPassword.KLPRSS_Val.value = '';
    }
    else if(changedData.NewPassword!=='') {
        const pswrd_data = getPswdrData(changedData.NewPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrd_data })
        if(response.result!=='')
            changedData.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationPassword.KLPRSS_Val.value = response.result;
    }

    return changedData;
}

function toRowNetAppScanScope(changedData) {
    let scanScope = []
    let scanExcludes = []
    for(let item of changedData.Areas){
        if(item.uiFields.Included === true){
            let scope = item;
            updateNetAppScopeFromUiFields(scope.ScanSettings.KLPRSS_Val, scope.uiFields);
            delete scope.uiFields;
            scanScope.push(scope);
        }
        else{
            const exclude = {
                AreaName:item.AreaName,
                AreaType:item.AreaType,
                __VersionInfo:{KLPRSS_Val:[1,0],KLPRSS_Mnd:false}
            };
            scanExcludes.push(exclude);
        }
    }
    changedData.ScanScope.KLPRSS_Val[0] = scanScope;
    changedData.ExcludedFromScanScope.KLPRSS_Val[0] = scanExcludes;
    return changedData;
}

function toRowNetAppScanScopeSetts(changedData) {
    let scanScope = []
    let scanExcludes = []
    for(let item of changedData.Areas){
        if(item.uiFields.Included === true){
            const scope = GetNetAppScanScopeSettings(item);
   //         let scope = hlp.ItemToLocalSettings(item);
   //         NetAppScopeSettingsFromUiFields(scope.ScanSettings,scope.uiFields);
   //         delete scope.uiFields
            scanScope.push(scope)
        }
        else{
            const exclude = {
                AreaName:item.AreaName.KLPRSS_Val,
                AreaType:item.AreaType.KLPRSS_Val,
                __VersionInfo:[1,0]
            }
            scanExcludes.push(exclude)
        }
    }
    return {
        ScanScope: scanScope,
        ExcludedFromScanScope: scanExcludes
    };
}

async function toRawNetAppSettPswrd(changedData, kscApi, hostId) {
    changedData.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationLogin.KLPRSS_Val = changedData.Login;
    changedData.NetAppConnectAsInfo.KLPRSS_Val[0].EnableNetAppImpersonation.KLPRSS_Val = changedData.Login==='' ? false : true;

    if(changedData.UseEmptyPassword){
        changedData.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationPassword.KLPRSS_Val.value = '';
    }
    else if(changedData.NewPassword!=='') {
        const pswrd_data = getPswdrData(changedData.NewPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataForHost ({szwHostId:hostId, pData:pswrd_data });
        if(response.result!=='')
            changedData.NetAppConnectAsInfo.KLPRSS_Val[0].NetAppImpersonationPassword.KLPRSS_Val.value = response.result;

    }

    return changedData;
}

module.exports = {
    toViewNetAppPswrd,
    toViewNetAppScanScope,
    toRawNetAppPswrd,
    toRowNetAppScanScope,
    toRowNetAppScanScopeSetts,
    toRawNetAppSettPswrd
}