const hlp = require('../../helpers');
const hlpPep = require('../../common/utils_pep.js');

const PolicyRulesUsageStrategy = { AddToLocal: 1, ReplaceLocal: 2 };

const toViewRules = (data) => {
    let jsonRules = {};
    let dummyRules = '';
    if (data.RuleUsageStrategy.KLPRSS_Val[0] === PolicyRulesUsageStrategy.ReplaceLocal) {
        jsonRules = hlp.parseBinXMLtoJSON(data.RuleCollection.KLPRSS_Val[0].value);
        dummyRules = data.AdditionalRuleCollection.KLPRSS_Val[0].value;
    } else {
        jsonRules = hlp.parseBinXMLtoJSON(data.AdditionalRuleCollection.KLPRSS_Val[0].value);
        dummyRules = data.RuleCollection.KLPRSS_Val[0].value;
    }

    return {
        Rules: hlpPep.toPepUiRules(jsonRules),
        jsonTempl: hlpPep.getDummyXmlRulesTemplate(jsonRules),
        dummyTempl: dummyRules
    }
}

const toViewProfileRules = (data) => {
    const jsonRules = hlp.parseBinXMLtoJSON(data.RuleCollectionProfile.KLPRSS_Val[0].value);
    return {
        Rules: hlpPep.toPepUiRules(jsonRules),
        jsonTempl: hlpPep.getDummyXmlRulesTemplate(jsonRules),
    }
}

const toViewSettingsRules = (binData) => {
    const jsonRules = hlp.parseBinXMLtoJSON(binData);
    return {
        Rules: hlpPep.toPepUiRules(jsonRules),
        jsonTempl: hlpPep.getDummyXmlRulesTemplate(jsonRules)
    }
}

const toRowRules = (changedData) => {
    let jsonRules = hlpPep.toJsonRules(changedData.uiFields.Rules, changedData.uiFields.jsonTempl);
    let packedRules = hlp.JSONtoBinXML(jsonRules);

    const bRulesMandatoried = changedData.RuleUsageStrategy.KLPRSS_Mnd ? true : false;
    // if Rules are mandatoried and set strategy AddToLocal we should save rules in additional_rule_collection
    if(changedData.RuleUsageStrategy.KLPRSS_Val[0]===PolicyRulesUsageStrategy.AddToLocal)
    {
        // in this case we save customized copy of policy - to allow father edition in UI without closing this dialog (BF: #2134690)
        changedData.AdditionalRuleCollection.KLPRSS_Mnd = bRulesMandatoried;
        changedData.AdditionalRuleCollection.KLPRSS_Val[0].value = packedRules;
        changedData.RuleCollection.KLPRSS_Mnd = false;
        changedData.RuleCollection.KLPRSS_Val[0].value = changedData.uiFields.dummyTempl;
    }
    else if(changedData.RuleUsageStrategy.KLPRSS_Val[0]===PolicyRulesUsageStrategy.ReplaceLocal)
    {
        changedData.RuleCollection.KLPRSS_Mnd = bRulesMandatoried;
        changedData.RuleCollection.KLPRSS_Val[0].value = packedRules;
        changedData.AdditionalRuleCollection.KLPRSS_Mnd = bRulesMandatoried;
        changedData.AdditionalRuleCollection.KLPRSS_Val[0].value = changedData.uiFields.dummyTempl;
    }
    delete changedData.uiFields;
    return changedData;
}

const toRowProfileRules = (changedData) => {
    const jsonRules = hlpPep.toJsonRules(changedData.uiFields.Rules, changedData.uiFields.jsonTempl);
    changedData.RuleCollectionProfile.KLPRSS_Val[0].value = hlp.JSONtoBinXML(jsonRules);

    delete changedData.uiFields;
    return changedData;
}

const toRowSettingsRules = (changedData) => {
    const jsonRules = hlpPep.toJsonRules(changedData.uiFields.Rules, changedData.uiFields.jsonTempl);
    return [{
        type: 'binary',
        value: hlp.JSONtoBinXML(jsonRules)
    }];
}

const GetDummyBinRules = () => {
    return [{
        type: 'binary',
        value: 'PEFwcGxpY2F0aW9uQ29udHJvbCBWZXJzaW9uPSIxIj48UnVsZUNvbGxlY3Rpb24gLz48L0FwcGxpY2F0aW9uQ29udHJvbD4='
    }];
}

module.exports = {
    toViewRules,
    toViewProfileRules,
    toViewSettingsRules,
    toRowRules,
    toRowProfileRules,
    toRowSettingsRules,
    GetDummyBinRules
}