const lo = require('lodash')
const xml2js = require('xml2js')
const { TextDecoder } =  require('util')

function serialize(data) {
    let data_copy = lo.cloneDeep(data)
    serializeRecursive(data_copy)
    return data_copy

    function serializeRecursive(data) {
        lo.each(data, (item, key) => {
            if (item == null) return
            if (item.KLPRSS_Val != null && item.KLPRSS_Mnd == null) {
                item.KLPRSS_Mnd = false
            }
            let isValueOriginal = item.type != null && item.value != null
            if (lo.isPlainObject(item) && !isValueOriginal) {
                data[key] = {
                    type: 'params',
                    value: item
                }
                serializeRecursive(data[key].value)
            }
            if (Array.isArray(item)) {
                serializeRecursive(item)
            }
        })
    }
}

function deserialize({ PxgRetVal }) {
    let result = PxgRetVal
    return mapRecusrive(result)

    function mapRecusrive(data) {
        if (data && typeof data === 'object') {
            if (Array.isArray(data)) {
                return data.map(mapRecusrive)
            } else {
                if (data.value !== undefined && data.type === 'params') {
                    data = mapRecusrive(data.value)
                } else {
                    Object.keys(data)
                        .forEach(property => {
                            data[property] = mapRecusrive(data[property])
                        })
                }
            }
        }
        return data
    }
}

function customToView(section, keys, initialState = {}) {
    return keys.reduce((acc, key) => {
        acc[key] = section[key];
        return acc;
    }, initialState);
}

function wrapper(val) {
    return {
        KLPRSS_Val:val,
        KLPRSS_Mnd:false
//        KLPRSS_ValLck: false,
    };
}

function wrap(data, wrapper) {
    let result = data;
    if (lo.isObject(data)) {
        result = Object.keys(data).reduce((res, key) => ({ ...res, [key]: wrap(data[key], wrapper) }), {});
    }
    return wrapper(result);
}

function GetMergedLocalSettingsKey(localSetts, policySetts, localKey, policyKey) {
    const wrapSetts = wrap(localSetts, wrapper);
    if(policyKey===undefined)
        policyKey = localKey;
    if(policySetts===undefined || policySetts=== {} ){
        return {KLPRSS_Mnd: false,  KLPRSS_ValLck: false,  KLPRSS_Val: [localSetts[localKey]] };
    }

    const bMandatory = lo.get(policySetts[policyKey], 'KLPRSS_Mnd', false);
    const bLocked = (lo.get(policySetts[policyKey], 'KLPRSS_ValLck', false)===true || bMandatory)
    let val = lo.get(policySetts[policyKey], 'KLPRSS_Val', [])
    if(!bMandatory){
        if(Array.isArray(val)){
            val[0] = UpdatePolicyKeyByLocalSettings(val[0], localSetts[localKey])
        } else{
            val = localSetts[localKey]
        }
    }

    return {
        KLPRSS_Mnd: bMandatory,
        KLPRSS_ValLck: bLocked,
//        KLPRSS_ValLckPolicy:lo.get(policySetts[policyKey], 'KLPRSS_ValLckPolicy',''),
//        KLPRSS_ValLckPrf:lo.get(policySetts[policyKey], 'KLPRSS_ValLckPrf',''),
//        KLPRSS_ValLckGroup:lo.get(policySetts[policyKey], 'KLPRSS_ValLckGroup',''),
        KLPRSS_Val: val
    };
}

function localSettingsToView(localSetts, policySetts, keys, initialState = {}) {
    return keys.reduce((acc, key) => {
        acc[key] = GetMergedLocalSettingsKey(localSetts, policySetts, key);
        return acc;
    }, initialState);
}

function ItemToPolicyView(item) {
    if(Array.isArray(item)) {
        return item.map(function(node) { return ItemToPolicyView(node); });
    }

    Object.keys(item).forEach(property => {
        if(item[property].KLPRSS_Val===undefined)   {
            item[property] = {
                KLPRSS_Val: (typeof item[property] !== 'object' || item[property].value!==undefined) ? item[property] : ItemToPolicyView(item[property])
            }
        }
    })
    return item;
}

function ItemToLocalSettings(item) {
    if(Array.isArray(item)) {
        return item.map(function(node) { return ItemToLocalSettings(node); });
    }

    Object.keys(item).forEach(property => {
        if(item[property].KLPRSS_Val!==undefined){
            if(property==='__VersionInfo'){
                item[property] = item[property].KLPRSS_Val;
            }
            else{
                let val = item[property].KLPRSS_Val;
                if(Array.isArray(val) && item[property].KLPRSS_Val[0]!== undefined){
                    val = item[property].KLPRSS_Val[0];
                }
                item[property] = (typeof val === 'object') ? ItemToLocalSettings(val) : val;
            }
        }
    })
    return item;
}

function UpdateItemProperty(itemPolicy, itemLocal, property){
    if (itemPolicy[property]===undefined || itemLocal[property]===undefined)
        return;
    if(property==='value'){
        itemPolicy[property] = itemLocal[property];
        return;
    }
    if (itemPolicy[property].KLPRSS_Val===undefined)
        return;

    if (typeof itemLocal[property] === 'object') {
        itemPolicy[property].KLPRSS_Val = UpdatePolicyKeyByLocalSettings(itemPolicy[property].KLPRSS_Val, itemLocal[property]);
    } else {
        itemPolicy[property].KLPRSS_Val = itemLocal[property];
    }
}

function UpdatePolicyKeyByLocalSettings(itemPolicy, itemLocal) {
    if(itemLocal===undefined)
        return itemPolicy;
    if(typeof itemPolicy !== 'object'){
        return itemLocal;
    }
    if(Array.isArray(itemPolicy)) {
        return itemLocal.map(function(node) { return ItemToPolicyView(node); });
    }
    Object.keys(itemLocal).forEach(property => {
        UpdateItemProperty(itemPolicy, itemLocal, property);
    })
    return itemPolicy;
}

function parseBinXMLtoJSON(rowXml) {
    let out = {}
    try {
        let buff = Buffer.from(rowXml, 'base64')
        let decodedXml = new TextDecoder('utf-8').decode(buff)
        xml2js.parseString(decodedXml, function (err, result) {
            out = err ? {} : result;
        });
    } catch (error) {
    }
    return out
}

function JSONtoBinXML(json) {
    try {
        const builder = new xml2js.Builder({ headless: true, renderOpts:{ pretty: true, indent:'', newline:''} });
        const rowXml = builder.buildObject(json);
        return Buffer.from(rowXml).toString('base64');
    } catch (error) {
        console.error(error)
    }
    return ''
}

module.exports = {
    serialize,
    deserialize,
    customToView,
    GetMergedLocalSettingsKey,
    localSettingsToView,
    ItemToPolicyView,
    ItemToLocalSettings,
    parseBinXMLtoJSON,
    JSONtoBinXML
}