import {CompoundObjectScanModeEnum, ScanTypeEnum, ProtectionLevel} from "../types";

const OdsActionEnum = {Cure: 1, Remove: 2, Recommended: 3, Report: 4, Quarantine: 5, Skip: 7, Postpone: 8};
const HSMScanActionEnum = { ScanEntireFile: 1, ScanResidentPortion: 2, Skip: 3};

const ProtectionLevelEnum = {
    HighSecurity: 1,
    Recommended: 2,
    HighSpeed: 3,
    Custom: 4,
    NonBlocking: 5
}

const CheckOdsLevel = (data) => {
    if( data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.AllObjects &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === true &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === true &&
        data.ScanPlainMail === true &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === false &&
        data.UseSizeLimit === false &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Cure &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Remove &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Quarantine &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Report &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === false &&
        data.ScanArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanSfxArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanMailBasesMode === CompoundObjectScanModeEnum.All &&
        data.ScanPlainMailMode === CompoundObjectScanModeEnum.All &&
        data.ScanPackedMode === CompoundObjectScanModeEnum.All &&
        data.ScanOleMode === CompoundObjectScanModeEnum.All &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.ScanEntireFile &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.maxProtect';
    }
    else if (
        data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.AllObjects &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === true &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === false &&
        data.ScanPlainMail === false &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === false &&
        data.UseSizeLimit === false &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Recommended &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Recommended &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Recommended &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Recommended &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === false &&
        data.ScanArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanSfxArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanPackedMode === CompoundObjectScanModeEnum.All &&
        data.ScanOleMode === CompoundObjectScanModeEnum.All &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.ScanResidentPortion &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.recomended';
    }
    else if (
        data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.ByFormat &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === false &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === false &&
        data.ScanPlainMail === false &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === true &&
        data.TimeLimit === 60 &&
        data.UseSizeLimit === true &&
        data.SizeLimit === 8388608 &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Cure &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Remove &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Quarantine &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Report &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === true &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.Skip &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.maxSpeed';
    }
    else if (
        data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.AllObjects &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === false &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === false &&
        data.ScanPlainMail === false &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === false &&
        data.UseSizeLimit === false &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Report &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Report &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Report &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Report &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === false &&
        data.ScanArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanSfxArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanPackedMode === CompoundObjectScanModeEnum.All &&
        data.ScanOleMode === CompoundObjectScanModeEnum.All &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.ScanResidentPortion &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.nonBlocking';
    }

    return 'protectionLevel.custom';
}

const ResetOdsLevel = (data, level) => {
    switch(level){
        case '1':
        case ProtectionLevel.HighSecurity:
            return {
                ...data,
                ScanObjects: true,
                ScanType: ScanTypeEnum.AllObjects,
                ScanSectors: true,
                ScanStreams: true,
                ScanArchived: true,
                ScanSfxArchived: true,
                ScanMailBases: true,
                ScanPlainMail: true,
                ScanPacked: true,
                ScanOle: true,
                UseTimeLimit: false,
                UseSizeLimit: false,
                InfectedFirstAction: { ...data.InfectedFirstAction, ActionType: OdsActionEnum.Cure },
                InfectedSecondAction: {...data.InfectedSecondAction, ActionType: OdsActionEnum.Remove },
                SuspiciousFirstAction: { ...data.SuspiciousFirstAction, ActionType: OdsActionEnum.Quarantine },
                SuspiciousSecondAction: {...data.SuspiciousSecondAction, ActionType: OdsActionEnum.Report },
                UseAdvancedActions: false,
                UseExcludeMasks: false,
                UseExcludeThreats: false,
                ScanOnlyNewAndChangedFiles: false,
                ScanArchivedMode: CompoundObjectScanModeEnum.All,
                ScanSfxArchivedMode: CompoundObjectScanModeEnum.All,
                ScanMailBasesMode: CompoundObjectScanModeEnum.All,
                ScanPlainMailMode: CompoundObjectScanModeEnum.All,
                ScanPackedMode: CompoundObjectScanModeEnum.All,
                ScanOleMode: CompoundObjectScanModeEnum.All,
                ScanHSMOptions: {...data.ScanHSMOptions, ActionType: HSMScanActionEnum.ScanEntireFile },
                DeleteArchives: false
            }
        case '2':
        case ProtectionLevel.Recommended:
            return {
                ...data,
                ScanObjects: true,
                ScanType: ScanTypeEnum.AllObjects,
                ScanSectors: true,
                ScanStreams: true,
                ScanArchived: true,
                ScanSfxArchived: true,
                ScanMailBases: false,
                ScanPlainMail: false,
                ScanPacked: true,
                ScanOle: true,
                UseTimeLimit: false,
                UseSizeLimit: false,
                InfectedFirstAction: { ...data.InfectedFirstAction, ActionType: OdsActionEnum.Recommended },
                InfectedSecondAction: {...data.InfectedSecondAction, ActionType: OdsActionEnum.Recommended },
                SuspiciousFirstAction: { ...data.SuspiciousFirstAction, ActionType: OdsActionEnum.Recommended },
                SuspiciousSecondAction: {...data.SuspiciousSecondAction, ActionType: OdsActionEnum.Recommended },
                UseAdvancedActions: false,
                UseExcludeMasks: false,
                UseExcludeThreats: false,
                ScanOnlyNewAndChangedFiles: false,
                ScanArchivedMode: CompoundObjectScanModeEnum.All,
                ScanSfxArchivedMode: CompoundObjectScanModeEnum.All,
                ScanMailBasesMode: CompoundObjectScanModeEnum.All,
                ScanPlainMailMode: CompoundObjectScanModeEnum.All,
                ScanPackedMode: CompoundObjectScanModeEnum.All,
                ScanOleMode: CompoundObjectScanModeEnum.All,
                ScanHSMOptions: {...data.ScanHSMOptions, ActionType: HSMScanActionEnum.ScanResidentPortion },
                DeleteArchives: false
            }
        case '3':
        case ProtectionLevel.HighSpeed:
            return {
                ...data,
                ScanObjects: true,
                ScanType: ScanTypeEnum.ByFormat,
                ScanSectors: true,
                ScanStreams: true,
                ScanArchived: false,
                ScanSfxArchived: true,
                ScanMailBases: false,
                ScanPlainMail: false,
                ScanPacked: true,
                ScanOle: true,
                UseTimeLimit: true,
                TimeLimit: 60,
                UseSizeLimit: true,
                SizeLimit: 8388608,
                InfectedFirstAction: { ...data.InfectedFirstAction, ActionType: OdsActionEnum.Cure },
                InfectedSecondAction: {...data.InfectedSecondAction, ActionType: OdsActionEnum.Remove },
                SuspiciousFirstAction: { ...data.SuspiciousFirstAction, ActionType: OdsActionEnum.Quarantine },
                SuspiciousSecondAction: {...data.SuspiciousSecondAction, ActionType: OdsActionEnum.Report },
                UseAdvancedActions: false,
                UseExcludeMasks: false,
                UseExcludeThreats: false,
                ScanOnlyNewAndChangedFiles: true,
 //               ScanArchivedMode: CompoundObjectScanModeEnum.All,
 //               ScanSfxArchivedMode: CompoundObjectScanModeEnum.All,
 //               ScanMailBasesMode: CompoundObjectScanModeEnum.All,
 //               ScanPlainMailMode: CompoundObjectScanModeEnum.All,
 //               ScanPackedMode: CompoundObjectScanModeEnum.All,
 //               ScanOleMode: CompoundObjectScanModeEnum.All,
                ScanHSMOptions: {...data.ScanHSMOptions, ActionType: HSMScanActionEnum.Skip },
                DeleteArchives: false
            }
        case '5':
        case ProtectionLevel.NonBlocking:
            return {
                ...data,
                ScanObjects: true,
                ScanType: ScanTypeEnum.AllObjects,
                ScanSectors: true,
                ScanStreams: true,
                ScanArchived: false,
                ScanSfxArchived: true,
                ScanMailBases: false,
                ScanPlainMail: false,
                ScanPacked: true,
                ScanOle: true,
                UseTimeLimit: false,
                UseSizeLimit: false,
                InfectedFirstAction: { ...data.InfectedFirstAction, ActionType: OdsActionEnum.Report },
                InfectedSecondAction: {...data.InfectedSecondAction, ActionType: OdsActionEnum.Report },
                SuspiciousFirstAction: { ...data.SuspiciousFirstAction, ActionType: OdsActionEnum.Report },
                SuspiciousSecondAction: {...data.SuspiciousSecondAction, ActionType: OdsActionEnum.Report },
                UseAdvancedActions: false,
                UseExcludeMasks: false,
                UseExcludeThreats: false,
                ScanOnlyNewAndChangedFiles: false,
                ScanArchivedMode: CompoundObjectScanModeEnum.All,
                ScanSfxArchivedMode: CompoundObjectScanModeEnum.All,
                ScanMailBasesMode: CompoundObjectScanModeEnum.All,
                ScanPlainMailMode: CompoundObjectScanModeEnum.All,
                ScanPackedMode: CompoundObjectScanModeEnum.All,
                ScanOleMode: CompoundObjectScanModeEnum.All,
                ScanHSMOptions: {...data.ScanHSMOptions, ActionType: HSMScanActionEnum.ScanResidentPortion },
                DeleteArchives: false
            }
    }

    return data;
}

module.exports = {
    ProtectionLevelEnum,
    CheckOdsLevel,
    ResetOdsLevel
}