const lo = require('lodash');
const fs = require('fs');
const path = require('path');
const buffer = require('buffer');
const { TextDecoder, TextEncoder } =  require('util');

const ConfigFileName = 'config.ini';
const SureInstallFeatures = 'Ods,AVProtection,RamDisk,AKIntegration,Shell';
const ManagedFeatures = [
    'Oas','ScriptChecker','Ksn','AppCtrl','DevCtrl','Firewall','Fim', 'AntiExploit','LogInspector',
    'PerfMonCounters','SnmpSupport','Shell','TrayApp'
];

const tzConfigures = [
    { "value": 0, "text": "AutomiqSoft_AlphaPlatform"},
    { "value": 1, "text": "BakerHughes_BentlyNevadaSystem"},
    { "value": 2, "text": "BOSCH_BIS"},
    { "value": 3, "text": "Emerson_OpenEnterprise"},
    { "value": 4, "text": "Emerson_Ovation"},
    { "value": 5, "text": "Emerson_Pipeline"},
    { "value": 6, "text": "PcVue_PcVue15"},
    { "value": 7, "text": "Rexroth_IndraWorks"},
    { "value": 8, "text": "Siemens_SIMATICPCS7-8.2SP1"},
    { "value": 9, "text": "Siemens_SIMATICPCS7-9.0SP2"},
    { "value": 10,"text": "SPEMT_VACSStandart"},
    { "value": 11,"text": "Tornado_Tornado-N"},
    { "value": 12,"text": "Valmet_DNA"},
    { "value": 13,"text": "Wonderware_System-Platform-2017"}
];

const getTzConfigureIndex = (name) => {
    for(item of tzConfigures){
        if(item.text === name)
            return item.value;
    }
    return 1000;
}

const getTzConfigureName = (index) => {
    for(item of tzConfigures){
        if(item.value === index)
            return item.text;
    }
    return 1000;
}

const defLoc = 'en-us';

const readFileKea = (loc) => {
    try {
        const filePath = path.resolve(__dirname, `../../assets/kea/${loc}.txt`);
        return fs.readFileSync(filePath, 'utf8');
    } catch (error) {
        console.log(error);
    }
    return '';
}

const getKeaEula = (loc) => {
    const eula = readFileKea(loc);
    return (eula==='' && loc!==defLoc) ? readFileKea(defLoc) : eula;
}

const AddFeature = (featuresList, feature) => {
    if (featuresList.length > 0)
        featuresList += ',';
    featuresList += feature;
    return featuresList;
}

function packageInfo() {
    let currentPackageId = -1

    const GetBoolKeyValue = (keys, key) => {
        return keys[key]!==undefined && keys[key]==='1' ? true : false;
    }

    const GetBoolCommandLineProp = (data, key, pref) => {
        if(data[key]===undefined)
            return '';
        const value = data[key]===true ? '1' : '0';
        return (pref + key + '=' + value);
    }

    const GetNumberCommandLineProp = (data, key, pref) => {
        return data[key] ? (pref + key + '=' + data[key].toString()) : '';
    }

    const GetKeys = (rowStr) => {
        let str = rowStr.replace('= ', '=').replace(' =', '=').replace('\r', '');
        let keys = {};
        while (true) {
            let indexA = str.indexOf(' ');
            if(indexA=== -1 && str.length > 0 )
                indexA = str.length;

            let token = str.substring(0, indexA);
            let indexB = token.indexOf('=');
            if(indexB!==-1 && token.length){
                if(token[indexB+1]==='"') {
                    indexA = str.indexOf('"', indexA + 2);
                    token = str.substring(0, indexA);
                    keys[token.substring(0, indexB)] = token.substring(indexB+2);
                }
                else
                    keys[token.substring(0, indexB)] = token.substring(indexB+1);
            }

            if(str.length===0 || indexA > str.length - 1)
                break;
            str = str.substring(indexA + 1);
        }
        return keys;
    }

    const toView = (keys, import_settings_path, loc) => {
        let data = {
            features: {  Ods:true,  AKIntegration: true },
            NOAVBASES: GetBoolKeyValue(keys, 'NOAVBASES'),
            PRESCAN: GetBoolKeyValue(keys, 'PRESCAN'),
            RUNRTP: GetBoolKeyValue(keys, 'RUNRTP'),
            ADDMSEXCLUSION: GetBoolKeyValue(keys, 'ADDMSEXCLUSION'),
            ADDKLEXCLUSION: GetBoolKeyValue(keys, 'ADDKLEXCLUSION'),
            SERVICE_AUTOSTART_DELAYED: GetBoolKeyValue(keys, 'SERVICE_AUTOSTART_DELAYED'),
 //           TZ_CONFIG_ID: TZ_CFG_NOUSED_ID,
            AMSI:GetBoolKeyValue(keys, 'AMSI'),
            SKIPCVEWINDOWS10: GetBoolKeyValue(keys, 'SKIPCVEWINDOWS10'),
            KEA_EULA: GetBoolKeyValue(keys, 'KEA_EULA'),
            INSTALLDIR: keys.INSTALLDIR!==undefined ? keys.INSTALLDIR : '',
            ImportPath: (import_settings_path!==undefined && import_settings_path!=='undefined') ? import_settings_path : '',
            LocEula:getKeaEula(loc)
        };
        let installed = keys['ADDLOCAL'].split(',');
        if(keys.ADD_ADV!==undefined && keys.ADD_ADV!==''){
            installed = installed.concat(keys.ADD_ADV.split(','));
        }

        for(let item of installed){
            data.features[item] = installed.includes(item);
        }
        if(data.KEA_EULA===false){
            data.features.Soyuz=false;
        }

        if( keys['TZ_CONFIG_ID']) {
            data.TZ_CONFIG_ID = getTzConfigureIndex(keys['TZ_CONFIG_ID']);
        }

        return data;
    }

    const toRow = (changedData) => {
        let decoded = GetBoolCommandLineProp(changedData, 'NOAVBASES', '');
        decoded += GetBoolCommandLineProp(changedData, 'PRESCAN', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'RUNRTP', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'ADDMSEXCLUSION', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'ADDKLEXCLUSION', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'SERVICE_AUTOSTART_DELAYED', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'AMSI', ' ');
        decoded += GetBoolCommandLineProp(changedData, 'SKIPCVEWINDOWS10', ' ');
//        if(changedData.TZ_CONFIG_ID !== TZ_CFG_NOUSED_ID) {
//            decoded += GetNumberCommandLineProp(changedData, 'TZ_CONFIG_ID', ' ');
//        }

/*
decoded += GetBoolCommandLineProp(changedData, 'KEA_EULA', ' ');
if(changedData.KEA_EULA===true) {
    decoded += ' KEA_PRIVACYPOLICY=1';
}
 */
        decoded += ' INSTALLDIR="' + changedData.INSTALLDIR + '"';

        let AddLocal = '';
        let Remove = '';
        for(let item of ManagedFeatures) {
            if(changedData.features[item]!==undefined && changedData.features[item]===true){
                AddLocal = AddFeature(AddLocal, item);
            }
            else{
                Remove = AddFeature(Remove, item);
            }
        }

        decoded += ' ADDLOCAL=' + SureInstallFeatures + ',' + AddLocal;
        if(Remove.length > 0)
            decoded += ' REMOVE=' + Remove;

        // update advanced features
        AddLocal = '';
        Remove = '';
        if(changedData.KEA_EULA!==undefined && changedData.KEA_EULA===true && changedData.features.Soyuz!==undefined && changedData.features.Soyuz===true){
            AddLocal = AddFeature(AddLocal, 'Soyuz');
        }
        else{
            Remove = AddFeature(Remove, 'Soyuz');
        }
        // ids
        if(changedData.features.IDS!==undefined && changedData.features.IDS===true){
            AddLocal = AddFeature(AddLocal, 'IDS');
        }
        else{
            Remove = AddFeature(Remove, 'IDS');
        }
        // regmon
        if(changedData.features.RegMonitor!==undefined && changedData.features.RegMonitor===true){
            AddLocal = AddFeature(AddLocal, 'RegMonitor');
        }
        else{
            Remove = AddFeature(Remove, 'RegMonitor');
        }

        if(AddLocal.length > 0)
            decoded += ' ADD_ADV=' + AddLocal;
        if(Remove.length > 0)
            decoded += ' REMOVE_ADV=' + Remove;

        decoded += ('\n' + changedData.ImportPath);
        return decoded;
    }

    const getPackageData = async ({kscApi, packageId, form}) => {
        currentPackageId = parseInt(packageId);
        try {
            let iniInfo = await kscApi.PackagesApi.ReadPkgCfgFile({nPackageId: currentPackageId, wstrFileName:ConfigFileName});
            let buff = Buffer.from(iniInfo.result, 'base64');
            let decoded = new TextDecoder('utf-16').decode(buff);
            const [command_line, import_settings_path] = decoded.split('\n');
            let keys = GetKeys(command_line);
            const loc = await kscApi.utils.getUILanguage();
            return toView (keys, import_settings_path, loc);
        } catch (error) {
            return { Error: true, ErrorMsg: error.message }
        }
        return {};
    }

    const savePackageData = async ({kscApi, modelData, form}) => {
        if( form!=='InstallPackage' )
            return;
        const decoded = toRow(modelData);
        let bytes = Buffer.from(decoded, 'utf-8');
        let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');
        let base64Str = bufStr16.toString('base64');
        await kscApi.PackagesApi.WritePkgCfgFile({nPackageId: currentPackageId, wstrFileName: ConfigFileName, pData: base64Str});
    }

    return {
        getPackageData,
        savePackageData
    }
}

module.exports = packageInfo